# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Utility functions for mutli-subjectParcellation:
this basically uses nipy io lib to perform IO opermation
in parcel definition processes
"""

import numpy as np
import os.path

from nibabel import load, save, Nifti1Image

from nipy.algorithms.clustering.utils import kmeans
from .discrete_domain import grid_domain_from_image
from .mroi import SubDomains
from ..mask import intersect_masks


def mask_parcellation(mask_images, nb_parcel, threshold=0, output_image=None):
    """ Performs the parcellation of a certain mask

    Parameters
    ----------
    mask_images: string or Nifti1Image or list of strings/Nifti1Images,
                 paths of mask image(s) that define(s) the common space.
    nb_parcel: int,
               number of desired parcels
    threshold: float, optional,
               level of intersection of the masks
    output_image: string, optional
                  path of the output image

    Returns
    -------
    wim: Nifti1Imagine instance,  representing the resulting parcellation
    """
    if isinstance(mask_images, basestring):
        mask = mask_images
    elif isinstance(mask_images, Nifti1Image):
        mask = mask_images
    else:
        # mask_images should be a list
        mask_data = intersect_masks(mask_images, threshold=0) > 0
        mask = Nifti1Image(mask_data.astype('u8'),
                           load(mask_images[0]).get_affine())

    domain = grid_domain_from_image(mask)
    cent, labels, J = kmeans(domain.coord, nb_parcel)
    sub_dom = SubDomains(domain, labels)
    # get id (or labels) image
    wim = sub_dom.to_image(fid='id', roi=True)
    return wim


def parcel_input(mask_images, learning_images, ths=.5, fdim=None):
    """Instantiating a Parcel structure from a give set of input

    Parameters
    ----------
    mask_images: string or Nifti1Image or list of strings/Nifti1Images,
                 paths of mask image(s) that define(s) the common space.
    learning_images: (nb_subject-) list of  (nb_feature-) list of strings,
                     paths of feature images used as input to the
                     parcellation procedure
    ths=.5: threshold to select the regions that are common across subjects.
            if ths = .5, thethreshold is half the number of subjects
    fdim: int, optional
          if nb_feature (the dimension of the data) used in subsequent analyses
          if greater than fdim,
          a PCA is perfomed to reduce the information in the data
          Byd efault, no reduction is performed

    Returns
    -------
    domain : discrete_domain.DiscreteDomain instance
        that stores the spatial information on the parcelled domain
    feature: (nb_subect-) list of arrays of shape (domain.size, fdim)
        feature information available to parcellate the data
    """
    nb_subj = len(learning_images)

    # get a group-level mask
    if isinstance(mask_images, basestring):
        mask = mask_images
    elif isinstance(mask_images, Nifti1Image):
        mask = mask_images
    else:
        # mask_images should be a list
        grp_mask = intersect_masks(mask_images, threshold=ths) > 0
        mask = Nifti1Image(grp_mask.astype('u8'),
                           load(mask_images[0]).get_affine())

    # build the domain
    domain = grid_domain_from_image(mask, nn=6)
    #nn = 6 for speed up and stability

    # load the functional data
    feature = []
    nbeta = len(learning_images[0])
    for s in range(nb_subj):
        if len(learning_images[s]) != nbeta:
            raise ValueError('Inconsistent number of dimensions')
        feature.append(np.array([domain.make_feature_from_image(b)
                                 for b in learning_images[s]]).T)

    # Possibly reduce the dimension of the functional data
    if (len(feature[0].shape) == 1) or (fdim is None):
        return domain, feature
    if fdim < feature[0].shape[1]:
        import numpy.linalg as nl
        subj = np.concatenate([s * np.ones(feature[s].shape[0]) \
                                   for s in range(nb_subj)])
        cfeature = np.concatenate(feature)
        cfeature -= np.mean(cfeature, 0)
        m1, m2, m3 = nl.svd(cfeature, 0)
        cfeature = np.dot(m1, np.diag(m2))
        cfeature = cfeature[:, 0:fdim]
        feature = [cfeature[subj == s] for s in range(nb_subj)]

    return domain, feature


def write_parcellation_images(Pa, template_path=None, indiv_path=None,
                              subject_id=None, swd=None):
    """ Write images that describe the spatial structure of the parcellation

    Parameters
    ----------
    Pa : MultiSubjectParcellation instance,
         the description of the parcellation
    template_path: string, optional,
                   path of the group-level parcellation image
    indiv_path: list of strings, optional
                paths of the individual parcellation images
    subject_id: list of strings of length Pa.nb_subj
                subject identifiers, used to infer the paths when not available
    swd: string, optional
         output directory used to infer the paths when these are not available
    """
    # argument check
    if swd == None:
        from tempfile import mkdtemp
        swd = mkdtemp()

    if subject_id == None:
        subject_id = ['subj_%04d' % s for s in range(Pa.nb_subj)]

    if len(subject_id) != Pa.nb_subj:
        raise ValueError('subject_id does not match parcellation')

    # If necessary, generate the paths
    if template_path is None:
        template_path = os.path.join(swd, "template_parcel.nii")
    if indiv_path is None:
        indiv_path = [os.path.join(swd, "parcel%s.nii" % subject_id[s])
                        for s in range(Pa.nb_subj)]

    # write the template image
    tlabs = Pa.template_labels.astype(np.int16)
    template = SubDomains(Pa.domain, tlabs)
    template_img = template.to_image(
        fid='id', roi=True, descrip='Intra-subject parcellation template')
    save(template_img, template_path)

    # write subject-related stuff
    for s in range(Pa.nb_subj):
        # write the individual label images
        labs = Pa.individual_labels[:, s]
        parcellation = SubDomains(Pa.domain, labs)
        parcellation_img = parcellation.to_image(
            fid='id', roi=True, descrip='Intra-subject parcellation')
        save(parcellation_img, indiv_path[s])


def parcellation_based_analysis(Pa, test_images, test_id='one_sample',
                                rfx_path=None, condition_id='', swd=None):
    """ This function computes parcel averages and RFX at the parcel-level

    Parameters
    ----------
    Pa: MultiSubjectParcellation instance
        the description of the parcellation
    test_images: (Pa.nb_subj-) list of paths
                 paths of images used in the inference procedure
    test_id: string, optional,
          if test_id=='one_sample', the one_sample statstic is computed
          otherwise, the parcel-based signal averages are returned
    rfx_path: string optional,
              path of the resulting one-sample test image, if applicable
    swd: string, optional
         output directory used to compute output path if rfx_path is not given
    condition_id: string, optional,
                  contrast/condition id  used to compute output path

    Returns
    -------
    test_data: array of shape(Pa.nb_parcel, Pa.nb_subj)
               the parcel-level signal average if test is not 'one_sample'
    prfx: array of shape(Pa.nb_parcel),
          the one-sample t-value if test_id is 'one_sample'
    """
    nb_subj = Pa.nb_subj

    # 1. read the test data
    if len(test_images) != nb_subj:
        raise ValueError('Inconsistent number of test images')

    test = np.array([Pa.domain.make_feature_from_image(ti)
                     for ti in test_images]).T
    test_data = Pa.make_feature('', np.array(test))

    if test_id is not 'one_sample':
        return test_data

    # 2. perform one-sample test
    # computation
    from ..utils.reproducibility_measures import ttest
    prfx = ttest(test_data)

    # Write the stuff
    template = SubDomains(Pa.domain, Pa.template_labels)
    template.set_roi_feature('prfx', prfx)
    wim = template.to_image('prfx', roi=True)
    hdr = wim.get_header()
    hdr['descrip'] = 'parcel-based random effects image (in t-variate)'
    if rfx_path is not None:
        save(wim, rfx_path)

    return prfx


def fixed_parcellation(mask_image, betas, nbparcel, nn=6, method='ward',
                          write_dir=None, mu=10., verbose=0, fullpath=None):
    """ Fixed parcellation of a given dataset

    Parameters
    ----------
    domain/mask_image
    betas: list of paths to activation images from the subject
    nbparcel, int : number fo desired parcels
    nn=6: number of nearest neighbors  to define the image topology
          (6, 18 or 26)
    method='ward': clustering method used, to be chosen among
                   'ward', 'gkm', 'ward_and-gkm'
                   'ward': Ward's clustering algorithm
                   'gkm': Geodesic k-means algorithm, random initialization
                   'gkm_and_ward': idem, initialized by Ward's clustering
    write_di: string, topional, write directory.
                    If fullpath is None too, then no file output.
    mu = 10., float: the relative weight of anatomical information
    verbose=0: verbosity mode
    fullpath=None, string,
                   path of the output image
                   If write_dir and fullpath are None then no file output.
                   If only fullpath is None then it is the write dir + a name
                   depending on the method.

    Notes
    -----
    Ward's method takes time (about 6 minutes for a 60K voxels dataset)

    Geodesic k-means is 'quick and dirty'

    Ward's + GKM is expensive but quite good

    To reduce CPU time, rather use nn=6 (especially with Ward)
    """
    from nipy.algorithms.graph.field import field_from_coo_matrix_and_data

    if method not in ['ward', 'gkm', 'ward_and_gkm', 'kmeans']:
        raise ValueError('unknown method')
    if nn not in [6, 18, 26]:
        raise ValueError('nn should be 6,18 or 26')

    # step 1: load the data ----------------------------
    # 1.1 the domain
    domain = grid_domain_from_image(mask_image, nn)

    if method is not 'kmeans':
        # 1.2 get the main cc of the graph
        # to remove the small connected components
        pass

    coord = domain.coord

    # 1.3 read the functional data
    beta = np.array([domain.make_feature_from_image(b) for b in betas])

    if len(beta.shape) > 2:
        beta = np.squeeze(beta)

    if beta.shape[0] != domain.size:
        beta = beta.T

    feature = np.hstack((beta, mu * coord / np.std(coord)))

    #step 2: parcellate the data ---------------------------

    if method is not 'kmeans':
        g = field_from_coo_matrix_and_data(domain.topology, feature)

    if method == 'kmeans':
        _, u, _ = kmeans(feature, nbparcel)

    if method == 'ward':
        u, _ = g.ward(nbparcel)

    if method == 'gkm':
        seeds = np.argsort(np.random.rand(g.V))[:nbparcel]
        _, u, _ = g.geodesic_kmeans(seeds)

    if method == 'ward_and_gkm':
        w, _ = g.ward(nbparcel)
        _, u, _ = g.geodesic_kmeans(label=w)

    lpa = SubDomains(domain, u)

    if verbose:
        var_beta = np.array(
            [np.var(beta[lpa.label == k], 0).sum() for k in range(lpa.k)])
        var_coord = np.array(
            [np.var(coord[lpa.label == k], 0).sum() for k in range(lpa.k)])
        size = lpa.get_size()
        vf = np.dot(var_beta, size) / size.sum()
        va = np.dot(var_coord, size) / size.sum()
        print nbparcel, "functional variance", vf, "anatomical variance", va

    # step3:  write the resulting label image
    if fullpath is not None:
        label_image = fullpath
    elif write_dir is not None:
        label_image = os.path.join(write_dir, "parcel_%s.nii" % method)
    else:
        label_image = None

    if label_image is not None:
        lpa_img = lpa.to_image(
            fid='id', roi=True, descrip='Intra-subject parcellation image')
        save(lpa_img, label_image)
        if verbose:
            print "Wrote the parcellation images as %s" % label_image

    return lpa



# -*- Mode: Python -*-  Not really, but the syntax is close enough

# :Author: 	Alexis Roche

# Include numpy defines via Cython
from numpy cimport ndarray, import_array, npy_intp

# Redefine size_t
ctypedef unsigned long int size_t


# Exports from fff_base.h
cdef extern from "fff_base.h":

    ctypedef enum fff_datatype:
        FFF_UNKNOWN_TYPE = -1, 
        FFF_UCHAR = 0,         
        FFF_SCHAR = 1,         
        FFF_USHORT = 2,        
        FFF_SSHORT = 3,        
        FFF_UINT = 4,          
        FFF_INT = 5,           
        FFF_ULONG = 6,         
        FFF_LONG = 7,          
        FFF_FLOAT = 8,         
        FFF_DOUBLE = 9
        
    unsigned int fff_nbytes(fff_datatype type) 

# Exports from fff_vector.h 
cdef extern from "fff_vector.h":
    
    ctypedef struct fff_vector:
        size_t size
        size_t stride
        int owner
        double* data

    fff_vector* fff_vector_new(size_t n)
    void fff_vector_delete(fff_vector* x)
    fff_vector fff_vector_view(double* data, size_t size, size_t stride)
    double fff_vector_get(fff_vector * x, size_t i)
    void fff_vector_set(fff_vector * x, size_t i, double a)
    void fff_vector_set_all(fff_vector * x, double a) 
    void fff_vector_scale(fff_vector * x, double a) 
    void fff_vector_add_constant(fff_vector * x, double a)
    void fff_vector_memcpy(fff_vector* x, fff_vector* y)
    void fff_vector_fetch(fff_vector* x, void* data, fff_datatype datatype, size_t stride) 
    void fff_vector_add(fff_vector * x, fff_vector * y)
    void fff_vector_sub(fff_vector * x, fff_vector * y)
    void fff_vector_mul(fff_vector * x, fff_vector * y)
    void fff_vector_div(fff_vector * x, fff_vector * y)
    long double fff_vector_sum(fff_vector* x)
    long double fff_vector_ssd(fff_vector* x, double* m, int fixed)
    long double fff_vector_sad(fff_vector* x, double m)
    double fff_vector_median(fff_vector* x)
    double fff_vector_quantile(fff_vector* x, double r, int interp)
    double fff_vector_wmedian_from_sorted_data(fff_vector* x_sorted, fff_vector* w)

# Exports from fff_matrix.h
cdef extern from "fff_matrix.h":

    ctypedef struct fff_matrix:
        size_t size1
        size_t size2
        size_t tda
        int owner
        double* data

    fff_matrix* fff_matrix_new(size_t nr, size_t nc)
    void fff_matrix_delete(fff_matrix* A)
    fff_matrix fff_matrix_view(double* data, size_t size1, size_t size2, size_t tda)
    double fff_matrix_get(fff_matrix* A, size_t i, size_t j)
    void fff_matrix_set_all(fff_matrix * A, double a)
    void fff_matrix_scale(fff_matrix * A, double a)
    void fff_matrix_add_constant(fff_matrix * A, double a)
    void fff_matrix_get_row(fff_vector * x, fff_matrix * A, size_t i)
    fff_matrix_get_col(fff_vector * x, fff_matrix * A, size_t j)
    fff_matrix_get_diag(fff_vector * x, fff_matrix * A)
    fff_matrix_set_row(fff_matrix * A, size_t i, fff_vector * x)
    fff_matrix_set_col(fff_matrix * A, size_t j, fff_vector * x)
    fff_matrix_set_diag(fff_matrix * A, fff_vector * x)
    void fff_matrix_transpose(fff_matrix* A, fff_matrix* B)
    void fff_matrix_memcpy(fff_matrix* A, fff_matrix* B)
    fff_matrix fff_matrix_view(double* data, size_t size1, size_t size2, size_t tda) 
    void fff_matrix_add (fff_matrix * A, fff_matrix * B)
    void fff_matrix_sub (fff_matrix * A, fff_matrix * B)
    void fff_matrix_mul_elements (fff_matrix * A, fff_matrix * B)
    void fff_matrix_div_elements (fff_matrix * A, fff_matrix * B)
                

# Exports from fff_array.h
cdef extern from "fff_array.h":

    ctypedef enum fff_array_ndims: 
        FFF_ARRAY_1D = 1,   
        FFF_ARRAY_2D = 2,   
        FFF_ARRAY_3D = 3,   
        FFF_ARRAY_4D = 4    
        
    ctypedef struct fff_array:
        fff_array_ndims ndims
        fff_datatype datatype
        size_t dimX
        size_t dimY
        size_t dimZ
        size_t dimT
        unsigned int offsetX
        unsigned int offsetY
        unsigned int offsetZ
        unsigned int offsetT
        void* data
        int owner

    fff_array* fff_array_new(fff_datatype datatype, size_t dimX, size_t dimY, size_t dimZ, size_t dimT)
    fff_array* fff_array_new1d(fff_datatype datatype, size_t dimX)
    fff_array* fff_array_new2d(fff_datatype datatype, size_t dimX, size_t dimY)
    fff_array* fff_array_new3d(fff_datatype datatype, size_t dimX, size_t dimY, size_t dimZ)
    void fff_array_delete(fff_array* thisone)
    double fff_array_get(fff_array* thisone, size_t x, size_t y, size_t z, size_t t)
    fff_array fff_array_get_block(fff_array* thisone,
                                  size_t x0, size_t x1, size_t fX,
                                  size_t y0, size_t y1, size_t fY,
                                  size_t z0, size_t z1, size_t fZ,
                                  size_t t0, size_t t1, size_t fT)
    fff_array fff_array_get_block1d(fff_array* thisone, size_t x0, size_t x1, size_t fX)
    fff_array fff_array_get_block2d(fff_array* thisone,
                                    size_t x0, size_t x1, size_t fX,
                                    size_t y0, size_t y1, size_t fY)
    fff_array fff_array_get_block3d(fff_array* thisone,
                                    size_t x0, size_t x1, size_t fX,
                                    size_t y0, size_t y1, size_t fY,
                                    size_t z0, size_t z1, size_t fZ)
    void fff_array_set(fff_array* thisone, size_t x, size_t y, size_t z, size_t t, double value)
    void fff_array_set1d(fff_array* thisone, size_t x, double value)
    void fff_array_set2d(fff_array* thisone, size_t x, size_t y, double value)
    void fff_array_set3d(fff_array* thisone, size_t x, size_t y, size_t z, double value)
    void fff_array_set_all(fff_array* thisone, double c)
    void fff_array_extrema(double* min, double* max, fff_array* thisone)
    void fff_array_copy(fff_array* ares,  fff_array* asrc)
    void fff_array_add(fff_array * x, fff_array * y)
    void fff_array_sub(fff_array * x, fff_array * y)
    void fff_array_div(fff_array * x, fff_array * y)
    void fff_array_mul(fff_array * x, fff_array * y)
    void fff_array_clamp(fff_array* ares, fff_array* asrc, double th, int* clamp)

# Exports from the Python fff wrapper
cdef extern from "fffpy.h":

    ctypedef struct fffpy_multi_iterator:
        int narr
        int axis
        fff_vector** vector 
        size_t index 
        size_t size 

    void fffpy_import_array()
    fff_vector* fff_vector_fromPyArray(ndarray x)
    ndarray fff_vector_toPyArray(fff_vector* y)
    ndarray fff_vector_const_toPyArray(fff_vector* y)
    fff_matrix* fff_matrix_fromPyArray(ndarray x)
    ndarray fff_matrix_toPyArray(fff_matrix* y)
    ndarray fff_matrix_const_toPyArray(fff_matrix* y)
    fff_array* fff_array_fromPyArray(ndarray x) 
    ndarray fff_array_toPyArray(fff_array* y) 
    fff_datatype fff_datatype_fromNumPy(int npy_type)
    int fff_datatype_toNumPy(fff_datatype fff_type)
    void fff_vector_fetch_using_NumPy(fff_vector* y, char* data, npy_intp stride, int type, int itemsize)
    fffpy_multi_iterator* fffpy_multi_iterator_new(int narr, int axis, ...) 
    void fffpy_multi_iterator_delete(fffpy_multi_iterator* thisone)
    void fffpy_multi_iterator_update(fffpy_multi_iterator* thisone)
    void fffpy_multi_iterator_reset(fffpy_multi_iterator* thisone)

    



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import numpy as np

import pylab as pl

from nipy.labs.utils.simul_multisubject_fmri_dataset import \
     surrogate_2d_dataset

pos = np.array([[10, 10],
                [14, 20],
                [23, 18]])
ampli = np.array([4, 5, 2])

# First generate some noiseless data
noiseless_data = surrogate_2d_dataset(n_subj=1, noise_level=0, spatial_jitter=0,
                                      signal_jitter=0, pos=pos, ampli=ampli)

pl.figure(figsize=(10, 3))
pl.subplot(1, 4, 1)
pl.imshow(noiseless_data[0])
pl.title('Noise-less data')

# Second, generate some group data, with default noise parameters
group_data = surrogate_2d_dataset(n_subj=3, pos=pos, ampli=ampli)

pl.subplot(1, 4, 2)
pl.imshow(group_data[0])
pl.title('Subject 1')
pl.subplot(1, 4, 3)
pl.title('Subject 2')
pl.imshow(group_data[1])
pl.subplot(1, 4, 4)
pl.title('Subject 3')
pl.imshow(group_data[2])



import sys

import numpy as np

from .. import intvol

from ..tests.test_intrinsic_volumes import nonintersecting_boxes, randorth

import numpy.testing as npt

def bench_lips3d():
    np.random.seed(20111001)
    phi = intvol.Lips3d
    EC3d = intvol.EC3d
    repeat = 4
    bx_sz = 60
    box1, box2, edge1, edge2 = nonintersecting_boxes((bx_sz,)*3)
    c = np.indices(box1.shape).astype(np.float)
    sys.stdout.flush()
    print "\nIntrinsic volumes 3D"
    print "--------------------"
    print 'Box1 %6.2f\n' % npt.measure('phi(c,box1)', repeat),
    print 'Box2 %6.2f\n' % npt.measure('phi(c, box2)', repeat),
    print 'Box1+2 %6.2f\n' % npt.measure('phi(c, box1 + box2)', repeat),
    d = np.random.standard_normal((10,) + (bx_sz,) * 3)
    print 'Box1+2 d %6.2f\n' % npt.measure('phi(d, box1 + box2)', repeat),
    U = randorth(p=6)[0:3]
    e = np.dot(U.T, c.reshape((c.shape[0], -1)))
    e.shape = (e.shape[0],) + c.shape[1:]
    print 'Box1+2 e %6.2f\n' % npt.measure('phi(e, box1 + box2)', repeat),
    print 'Box1+2 EC %6.2f\n' % npt.measure('EC3d(box1 + box2)', repeat),
    sys.stdout.flush()


def bench_lips2d():
    np.random.seed(20111001)
    phi = intvol.Lips2d
    EC2d = intvol.EC2d
    repeat = 4
    bx_sz = 500
    box1, box2, edge1, edge2 = nonintersecting_boxes((bx_sz,)*2)
    c = np.indices(box1.shape).astype(np.float)
    sys.stdout.flush()
    print "\nIntrinsic volumes 2D"
    print "--------------------"
    print 'Box1 %6.2f\n' % npt.measure('phi(c,box1)', repeat),
    print 'Box2 %6.2f\n' % npt.measure('phi(c, box2)', repeat),
    print 'Box1+2 %6.2f\n' % npt.measure('phi(c, box1 + box2)', repeat),
    d = np.random.standard_normal((10,) + (bx_sz,) * 2)
    print 'Box1+2 d %6.2f\n' % npt.measure('phi(d, box1 + box2)', repeat),
    U = randorth(p=6)[0:2]
    e = np.dot(U.T, c.reshape((c.shape[0], -1)))
    e.shape = (e.shape[0],) + c.shape[1:]
    print 'Box1+2 e %6.2f\n' % npt.measure('phi(e, box1 + box2)', repeat),
    print 'Box1+2 EC %6.2f\n' % npt.measure('EC2d(box1 + box2)', repeat),
    sys.stdout.flush()


def bench_lips1d():
    np.random.seed(20111001)
    phi = intvol.Lips1d
    EC1d = intvol.EC1d
    repeat = 4
    bx_sz = 100000
    box1, box2, edge1, edge2 = nonintersecting_boxes((bx_sz,))
    c = np.indices(box1.shape).astype(np.float)
    sys.stdout.flush()
    print "\nIntrinsic volumes 1D"
    print "--------------------"
    print 'Box1 %6.2f\n' % npt.measure('phi(c,box1)', repeat),
    print 'Box2 %6.2f\n' % npt.measure('phi(c, box2)', repeat),
    print 'Box1+2 %6.2f\n' % npt.measure('phi(c, box1 + box2)', repeat),
    d = np.random.standard_normal((10, bx_sz))
    print 'Box1+2 d %6.2f\n' % npt.measure('phi(d, box1 + box2)', repeat),
    U = randorth(p=6)[0:1]
    e = np.dot(U.T, c.reshape((c.shape[0], -1)))
    e.shape = (e.shape[0],) + c.shape[1:]
    print 'Box1+2 e %6.2f\n' % npt.measure('phi(e, box1 + box2)', repeat),
    print 'Box1+2 EC %6.2f\n' % npt.measure('EC1d(box1 + box2)', repeat),
    sys.stdout.flush()



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
''' Interfaces to SPM '''
from __future__ import with_statement

import os

import numpy as np

from scipy.io import savemat

from nibabel import load
from nibabel.onetime import setattr_on_read
from nibabel.tmpdirs import InTemporaryDirectory

from .matlab import run_matlab_script


class SpmInfo(object):
    @setattr_on_read
    def spm_path(self):
        with InTemporaryDirectory() as tmpdir:
            run_matlab_script("""
spm_path = spm('dir');
fid = fopen('spm_path.txt', 'wt');
fprintf(fid, '%s', spm_path);
fclose(fid);
""")
            spm_path = open('spm_path.txt', 'rt').read()
        return spm_path

spm_info = SpmInfo()

                
def make_job(jobtype, jobname, contents):
    return {'jobs':[{jobtype:[{jobname:contents}]}]}


# XXX this should be moved into a matdict class or something
def fltcols(vals):
    ''' Trivial little function to make 1xN float vector '''
    return np.atleast_2d(np.array(vals, dtype=float))


def run_jobdef(jobdef):
    with InTemporaryDirectory():
        savemat('pyjobs.mat', jobdef)
        run_matlab_script("""
load pyjobs;
spm_jobman('run', jobs);
""")


def scans_for_fname(fname):
    img = load(fname)
    n_scans = img.shape[3]
    scans = np.zeros((n_scans, 1), dtype=object)
    for sno in range(n_scans):
        scans[sno] = '%s,%d' % (fname, sno+1)
    return scans


def scans_for_fnames(fnames):
    n_sess = len(fnames)
    sess_scans = np.zeros((1,n_sess), dtype=object)
    for sess in range(n_sess):
        sess_scans[0,sess] = scans_for_fname(fnames[sess])
    return sess_scans


def fname_presuffix(fname, prefix='', suffix='', use_ext=True):
    pth, fname = os.path.split(fname)
    fname, ext = os.path.splitext(fname)
    if not use_ext:
        ext = ''
    return os.path.join(pth, prefix+fname+suffix+ext)


def fnames_presuffix(fnames, prefix='', suffix=''):
    f2 = []
    for fname in fnames:
        f2.append(fname_presuffix(fname, prefix, suffix))
    return f2





# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
def configuration(parent_package='',top_path=None):
    from numpy.distutils.misc_util import Configuration
    config = Configuration('image', parent_package, top_path)

    config.add_subpackage('tests')

    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
# Initialization for diagnostics package

from .timediff import time_slice_diffs
from .tsdiffplot import plot_tsdiffs, plot_tsdiffs_image
from .screens import screen
from ..utils import pca



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function
__doc__ = """
Example of a one-sample t-test using the GLM formalism.
This script takes individual contrast images and masks and runs a simple GLM.
This can be readily generalized to any design matrix.

This particular example shows the statical map of a contrast
related to a computation task
(subtraction of computation task minus sentence reading/listening).

Needs matplotlib.

Author : Bertrand Thirion, 2012
"""
print(__doc__)

#autoindent
from os import mkdir, getcwd, path

import numpy as np

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise RuntimeError("This script needs the matplotlib library")

from nibabel import load, concat_images, save, Nifti1Image

from nipy.labs.mask import intersect_masks
from nipy.modalities.fmri.glm import FMRILinearModel
from nipy.labs.viz import plot_map, cm

# Local import
from get_data_light import DATA_DIR, get_second_level_dataset

# Get the data
n_subjects = 12
n_beta = 29
data_dir = path.join(DATA_DIR, 'group_t_images')
mask_images = [path.join(data_dir, 'mask_subj%02d.nii' % n)
               for n in range(n_subjects)]

betas = [path.join(data_dir, 'spmT_%04d_subj_%02d.nii' % (n_beta, n))
         for n in range(n_subjects)]

missing_files = np.array([not path.exists(m) for m in mask_images + betas])
if missing_files.any():
    get_second_level_dataset()

write_dir = path.join(getcwd(), 'results')
if not path.exists(write_dir):
    mkdir(write_dir)

# Compute a population-level mask as the intersection of individual masks
grp_mask = Nifti1Image(intersect_masks(mask_images).astype(np.int8),
                       load(mask_images[0]).get_affine())

# concatenate the individual images
first_level_image = concat_images(betas)

# set the model
design_matrix = np.ones(len(betas))[:, np.newaxis]  # only the intercept
grp_model = FMRILinearModel(first_level_image, design_matrix, grp_mask)

# GLM fitting using ordinary least_squares
grp_model.fit(do_scaling=False, model='ols')

# specify and estimate the contrast
contrast_val = np.array(([[1]]))  # the only possible contrast !
z_map, = grp_model.contrast(contrast_val, con_id='one_sample', output_z=True)

# write the results
save(z_map, path.join(write_dir, 'one_sample_z_map.nii'))

# look at the result
vmax = max(- z_map.get_data().min(), z_map.get_data().max())
vmin = - vmax
plot_map(z_map.get_data(), z_map.get_affine(),
         cmap=cm.cold_hot,
         vmin=vmin,
         vmax=vmax,
         threshold=3.,
         black_bg=True)
plt.savefig(path.join(write_dir, '%s_z_map.png' % 'one_sample'))
plt.show()
print("Wrote all the results in directory %s" % write_dir)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Frame of reference/coordinates package.

Mathematical model
==================
The idea of a chart :lm:`\\phi` : I{U} S{sub} I{M} S{->} B{R}^k
on a "manifold" I{M}.  For a chart both input (I{M}) and output coordinates
(B{R}^k) must be defined and a map relating the two coordinate systems.

Description
===========
The modules in this package contains classes which define the space in which an
image exists and also functions for manipulating and traversing this space.

The basic class which defines an image space is a CoordinateMap (coordinate_map.py). A 
CoordinateMap consists of an input CoordinateSystem (coordinate_system.py), an
output CoordinateSystem, and a mapping which converts points in the
input space to points in the output space.

A `CoordinateSystem` consists of a set of ordered `Coordinate` objects. Each
Coordinate has a name and a builtin dtype.

The typical use of a `CoordinateMap` is to define how voxels in an `Image`
(core.image.__init__.py) object's raw data map into real space. 

`Image` traversal is general done in terms of the underlying coordinate_map, and a number of
iterators are provided to traverse points in the coordinate_map (iterators.py). Access to
available iterators is done through the CoordinateMap interface, rather than 
accessing the iterator classes directly. 

The other common image access method is to take slices through the coordinate_map. In 
slices.py functions are presented which will return a `CoordinateMap` representing
a single slice through a larger coordinate_map.

"""
__docformat__ = 'restructuredtext'

from . import coordinate_system
from . import coordinate_map
from . import slices

__all__ = ["coordinate_system", "coordinate_map", 
           "slices"]

from nipy.testing import Tester
test = Tester().test
bench = Tester().bench



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function # Python 2/3 compatibility
__doc__ = """
Example of script to analyse the reproducibility in group studies using a
bootstrap procedure

Needs matplotlib

Author: Bertrand Thirion, 2005-2009
"""
print(__doc__)

import numpy as np

# Scipy stats needed for thresholding
import scipy.stats as st

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise RuntimeError("This script needs the matplotlib library")

import nipy.labs.utils.simul_multisubject_fmri_dataset as simul
from nipy.labs.utils.reproducibility_measures import (
    voxel_reproducibility, cluster_reproducibility, map_reproducibility,
    peak_reproducibility)
from nipy.labs.spatial_models.discrete_domain import (
    grid_domain_from_binary_array)

###############################################################################
# Generate the data
n_subj = 105
shape = (60, 60)
pos = np.array([[12, 14],
                [20, 20],
                [30, 20]])
ampli = np.array([2.5, 3.5, 3])
betas = simul.surrogate_2d_dataset(n_subj=n_subj, shape=shape, pos=pos,
                                     ampli=ampli, width=5.0)

n_vox = np.prod(shape)
# set the variance at 1 everywhere
func = np.reshape(betas, (n_subj, n_vox)).T
var = np.ones((n_vox, n_subj))
domain = grid_domain_from_binary_array(np.ones((shape[0], shape[1], 1)))

###############################################################################
# Run reproducibility analysis

ngroups = 10
thresholds = np.arange(.5, 6., .5)
sigma = 2.0
csize = 10
niter = 10
method = 'crfx'
verbose = 0

# do not use permutations
swap = False

kap = []
clt = []
pk = []
sens = []
for threshold in thresholds:
    kwargs={'threshold': threshold, 'csize': csize}
    kappa = []
    cls = []
    sent = []
    peaks = []
    for i in range(niter):
        k = voxel_reproducibility(func, var, domain, ngroups, method, swap,
                                  verbose, **kwargs)
        kappa.append(k)
        cld = cluster_reproducibility(func, var, domain, ngroups, sigma, method,
                                      swap, verbose, **kwargs)
        cls.append(cld)
        peak = peak_reproducibility(func, var, domain, ngroups, sigma, method,
                                    swap, verbose, **kwargs)
        peaks.append(peak)
        seni = map_reproducibility(func, var, domain, ngroups, method, True,
                                   verbose, threshold=threshold,
                                   csize=csize).mean()/ngroups
        sent.append(seni)
    sens.append(np.array(sent))
    kap.append(np.array(kappa))
    clt.append(np.array(cls))
    pk.append(np.array(peaks))

###############################################################################
# Visualize the results
aux = st.norm.sf(thresholds)

a = plt.figure(figsize=(11, 6))
plt.subplot(1, 3, 1)
plt.boxplot(kap)
plt.title('voxel-level \n reproducibility', fontsize=12)
plt.xticks(range(1, 1 + len(thresholds)), thresholds, fontsize=9)
plt.xlabel('threshold')
plt.subplot(1, 3, 2)
plt.boxplot(clt)
plt.title('cluster-level \n reproducibility', fontsize=12)
plt.xticks(range(1, 1 + len(thresholds)), thresholds, fontsize=9)
plt.xlabel('threshold')
plt.subplot(1, 3, 3)
plt.boxplot(pk, notch=1)
plt.title('peak-level \n reproducibility', fontsize=12)
plt.xticks(range(1, 1 + len(thresholds)), thresholds, fontsize=9)
plt.xlabel('threshold')

plt.figure()
for q, threshold in enumerate(thresholds):
    plt.subplot(3, len(thresholds) / 3 + 1, q + 1)
    rmap = map_reproducibility(func, var, domain, ngroups, method, verbose,
                               threshold=threshold, csize=csize)
    rmap = np.reshape(rmap, shape)
    plt.imshow(rmap, interpolation=None, vmin=0, vmax=ngroups)
    plt.title('threshold: %g' % threshold, fontsize=10)
    plt.axis('off')


plt.suptitle('Map reproducibility for different thresholds')
plt.show()



"""
The estimators for the intrinsic volumes appearing in this module
were partially supported by NSF grant DMS-0405970.

Taylor, J.E. & Worsley, K.J. (2007). "Detecting sparse signal in random fields,
   with an application to brain mapping."
   Journal of the American Statistical Association, 102(479):913-928.

"""
cimport cython

import numpy as np
cimport numpy as np

from scipy.sparse import dok_matrix

# Array helper
from nipy.utils.arrays import strides_from

# local imports
from utils import cube_with_strides_center, join_complexes


cdef double PI = np.pi


cdef extern from "math.h" nogil:
    double floor(double x)
    double sqrt(double x)
    double fabs(double x)
    double log2(double x)
    double acos(double x)
    bint isnan(double x)


cpdef double mu3_tet(double D00, double D01, double D02, double D03,
                     double D11, double D12, double D13,
                     double D22, double D23,
                     double D33) nogil:
  """ Compute the 3rd intrinsic volume of a tetrahedron.

  3rd intrinsic volume (just volume in this case) of a tetrahedron with
  coordinates implied by dot products below.

  Parameters
  ----------
  D00 : float
    If ``cv0`` is a 3-vector of coordinates for the first vertex, `D00` is
    ``cv0.dot(cv0)``
  D01 : float
    ``cv0.dot(cv1)`` where ``cv1`` is the coordinates for the second
    vertex.
  D02 : float
    ``cv0.dot(cv2)``
  D03 : float
    ``cv0.dot(cv3)``
  D11 : float
    ``cv1.dot(cv1)``
  D12 : float
    ``cv1.dot(cv2)``
  D13 : float
    ``cv1.dot(cv3)``
  D22 : float
    ``cv2.dot(cv2)``
  D23 : float
    ``cv2.dot(cv2)``
  D33 : float
    ``cv3.dot(cv3)``

  Returns
  -------
  mu3 : float
    volume of tetrahedron
  """
  cdef double C00, C01, C02, C11, C12, C22, v2
  C00 = D00 - 2*D03 + D33
  C01 = D01 - D13 - D03 + D33
  C02 = D02 - D23 - D03 + D33
  C11 = D11 - 2*D13 + D33
  C12 = D12 - D13 - D23 + D33
  C22 = D22 - 2*D23 + D33
  v2 = (C00 * (C11 * C22 - C12 * C12) -
        C01 * (C01 * C22 - C02 * C12) +
        C02 * (C01 * C12 - C11 * C02))
  # Rounding errors near 0 cause NaNs
  if v2 <= 0:
      return 0
  return sqrt(v2) / 6.


cpdef double mu2_tet(double D00, double D01, double D02, double D03,
                     double D11, double D12, double D13,
                     double D22, double D23,
                     double D33) nogil:
  """ Compute the 2nd intrinsic volume of tetrahedron

  2nd intrinsic volume (half the surface area) of a tetrahedron with coordinates
  implied by dot products below.

  Parameters
  ----------
  D00 : float
    If ``cv0`` is a 3-vector of coordinates for the first vertex, `D00` is
    ``cv0.dot(cv0)``
  D01 : float
    ``cv0.dot(cv1)`` where ``cv1`` is the coordinates for the second
    vertex.
  D02 : float
    ``cv0.dot(cv2)``
  D03 : float
    ``cv0.dot(cv3)``
  D11 : float
    ``cv1.dot(cv1)``
  D12 : float
    ``cv1.dot(cv2)``
  D13 : float
    ``cv1.dot(cv3)``
  D22 : float
    ``cv2.dot(cv2)``
  D23 : float
    ``cv2.dot(cv2)``
  D33 : float
    ``cv3.dot(cv3)``

  Returns
  -------
  mu2 : float
    Half tetrahedron surface area
  """
  cdef double mu = 0
  mu += mu2_tri(D00, D01, D02, D11, D12, D22)
  mu += mu2_tri(D00, D02, D03, D22, D23, D33)
  mu += mu2_tri(D11, D12, D13, D22, D23, D33)
  mu += mu2_tri(D00, D01, D03, D11, D13, D33)
  return mu * 0.5


cpdef double mu1_tet(double D00, double D01, double D02, double D03,
                     double D11, double D12, double D13,
                     double D22, double D23,
                     double D33) nogil:
  """ Return 3rd intrinsic volume of tetrahedron

  Compute the 3rd intrinsic volume (sum of external angles * edge
  lengths) of a tetrahedron for which the input arguments represent the
  coordinate dot products of the vertices.

  Parameters
  ----------
  D00 : float
    If ``cv0`` is a 3-vector of coordinates for the first vertex, `D00` is
    ``cv0.dot(cv0)``
  D01 : float
    ``cv0.dot(cv1)`` where ``cv1`` is the coordinates for the second
    vertex.
  D02 : float
    ``cv0.dot(cv2)``
  D03 : float
    ``cv0.dot(cv3)``
  D11 : float
    ``cv1.dot(cv1)``
  D12 : float
    ``cv1.dot(cv2)``
  D13 : float
    ``cv1.dot(cv3)``
  D22 : float
    ``cv2.dot(cv2)``
  D23 : float
    ``cv2.dot(cv2)``
  D33 : float
    ``cv3.dot(cv3)``

  Returns
  -------
  mu1 : float
    3rd intrinsic volume of tetrahedron
  """
  cdef double mu
  mu = 0
  mu += _mu1_tetface(D00, D01, D11, D02, D03, D12, D13, D22, D23, D33)
  mu += _mu1_tetface(D00, D02, D22, D01, D03, D12, D23, D11, D13, D33)
  mu += _mu1_tetface(D00, D03, D33, D01, D02, D13, D23, D11, D12, D22)
  mu += _mu1_tetface(D11, D12, D22, D01, D13, D02, D23, D00, D03, D33)
  mu += _mu1_tetface(D11, D13, D33, D01, D12, D03, D23, D00, D02, D22)
  mu += _mu1_tetface(D22, D23, D33, D02, D12, D03, D13, D00, D01, D11)
  return mu


cdef inline double limited_acos(double val) nogil:
    """ Check for -1 <= val <= 1 before returning acos(val)

    Avoids nan values from small rounding errors
    """
    if val >= 1:
        return 0
    elif val <= -1:
        return PI
    return acos(val)


@cython.cdivision(True)
cpdef double _mu1_tetface(double Ds0s0,
                          double Ds0s1,
                          double Ds1s1,
                          double Ds0t0,
                          double Ds0t1,
                          double Ds1t0,
                          double Ds1t1,
                          double Dt0t0,
                          double Dt0t1,
                          double Dt1t1) nogil:
    cdef double A00, A01, A02, A11, A12, A22, np_len, a, acosval
    cdef double length, norm_proj0, norm_proj1, inner_prod_proj

    A00 = Ds1s1 - 2 * Ds0s1 + Ds0s0
    # all norms divided by this value, leading to NaN value for output, for
    # values <= 0
    if A00 <= 0:
      return 0
    A11 = Dt0t0 - 2 * Ds0t0 + Ds0s0
    A22 = Dt1t1 - 2 * Ds0t1 + Ds0s0
    A01 = Ds1t0 - Ds0t0 - Ds0s1 + Ds0s0
    A02 = Ds1t1 - Ds0t1 - Ds0s1 + Ds0s0
    A12 = Dt0t1 - Ds0t0 - Ds0t1 + Ds0s0
    length = sqrt(A00)
    norm_proj0 = A11 - A01 * A01 / A00
    norm_proj1 = A22 - A02 * A02 / A00
    inner_prod_proj = A12 - A01 * A02 / A00
    np_len = norm_proj0 * norm_proj1
    if np_len <= 0: # would otherwise lead to NaN return value
        return 0
    # hedge for small rounding errors above 1 and below -1
    acosval = limited_acos(inner_prod_proj / sqrt(np_len))
    a = (PI - acosval) * length / (2 * PI)
    return a


cpdef double mu2_tri(double D00, double D01, double D02,
                     double D11, double D12,
                     double D22) nogil:
  """ Compute the 2nd intrinsic volume of triangle

  2nd intrinsic volume (just area in this case) of a triangle with coordinates
  implied by the dot products below.

  Parameters
  ----------
  D00 : float
    If ``cv0`` is a 3-vector of coordinates for the first vertex, `D00` is
    ``cv0.dot(cv0)``
  D01 : float
    ``cv0.dot(cv1)`` where ``cv1`` is the coordinates for the second
    vertex.
  D02 : float
    ``cv0.dot(cv2)``
  D11 : float
    ``cv1.dot(cv1)``
  D12 : float
    ``cv1.dot(cv2)``
  D22 : float
    ``cv2.dot(cv2)``

  Returns
  -------
  mu2 : float
    area of triangle
  """
  cdef double C00, C01, C11, L
  C00 = D11 - 2*D01 + D00
  C01 = D12 - D01 - D02 + D00
  C11 = D22 - 2*D02 + D00
  L = C00 * C11 - C01 * C01
  # Negative area appeared to result from floating point errors on PPC
  if L < 0:
      return 0.0
  return sqrt(L) * 0.5


cpdef double mu1_tri(double D00, double D01, double D02,
                     double D11, double D12,
                     double D22) nogil:
  """ Compute the 1st intrinsic volume of triangle

  1st intrinsic volume (1/2 the perimeter) of a triangle with coordinates
  implied by the dot products below.

  Parameters
  ----------
  D00 : float
    If ``cv0`` is a 3-vector of coordinates for the first vertex, `D00` is
    ``cv0.dot(cv0)``
  D01 : float
    ``cv0.dot(cv1)`` where ``cv1`` is the coordinates for the second
    vertex.
  D02 : float
    ``cv0.dot(cv2)``
  D11 : float
    ``cv1.dot(cv1)``
  D12 : float
    ``cv1.dot(cv2)``
  D22 : float
    ``cv2.dot(cv2)``

  Returns
  -------
  mu1 : float
    1/2 perimeter of triangle
  """
  cdef double mu = 0
  mu += mu1_edge(D00, D01, D11)
  mu += mu1_edge(D00, D02, D22)
  mu += mu1_edge(D11, D12, D22)
  return mu * 0.5


cpdef double mu1_edge(double D00, double D01, double D11) nogil:
  """ Compute the 1st intrinsic volume (length) of line segment

  Length of a line segment with vertex coordinates implied by dot products
  below.

  Parameters
  ----------
  D00 : float
    If ``cv0`` is a 3-vector of coordinates for the line start, `D00` is
    ``cv0.dot(cv0)``
  D01 : float
    ``cv0.dot(cv1)`` where ``cv1`` is the coordinates for the line end.
  D11 : float
    ``cv1.dot(cv1)``

  Returns
  -------
  mu0 : float
    length of line segment
  """
  return sqrt(D00 - 2*D01 + D11)


def EC3d(mask):
    """ Compute Euler characteristic of region within `mask`

    Given a 3d `mask`, compute the 0th intrinsic volume (Euler characteristic)
    of the masked region. The region is broken up into tetrahedra / triangles /
    edges / vertices, which are included based on whether all voxels in the
    tetrahedron / triangle / edge / vertex are in the mask or not.

    Parameters
    ----------
    mask : ndarray((i,j,k), np.int)
         Binary mask determining whether or not a voxel is in the mask.

    Returns
    -------
    mu0 : int
        Euler characteristic

    Notes
    -----
    The array mask is assumed to be binary. At the time of writing, it is not
    clear how to get cython to use np.bool arrays.

    The 3d cubes are triangulated into 6 tetrahedra of equal volume, as
    described in the reference below.

    References
    ----------
    Taylor, J.E. & Worsley, K.J. (2007). "Detecting sparse signal in random fields,
      with an application to brain mapping."
      Journal of the American Statistical Association, 102(479):913-928.
    """
    if not set(np.unique(mask)).issubset([0,1]):
      raise ValueError('mask should be filled with 0/1 '
                       'values, but be of type np.int')
    cdef:
        # c-level versions of the array
        np.ndarray[np.intp_t, ndim=3] mask_c
        # 'flattened' mask (1d array)
        np.ndarray[np.intp_t, ndim=1] fpmask
        # d3 and d4 are lists of triangles and tetrahedra
        # associated to particular voxels in the cuve
        np.ndarray[np.intp_t, ndim=2] d2
        np.ndarray[np.intp_t, ndim=2] d3
        np.ndarray[np.intp_t, ndim=2] d4
        # scalars
        np.npy_intp i, j, k, l, s0, s1, s2, ds2, ds3, ds4, index, m, nvox
        np.npy_intp ss0, ss1, ss2 # strides
        np.npy_intp v0, v1, v2, v3 # vertices
        np.npy_intp l0 = 0

    mask_c = mask

    pmask_shape = np.array(mask.shape) + 1
    pmask = np.zeros(pmask_shape, np.int)
    pmask[:-1,:-1,:-1] = mask_c

    s0, s1, s2 = (pmask.shape[0], pmask.shape[1], pmask.shape[2])

    fpmask = pmask.reshape(-1)
    cdef:
        np.ndarray[np.intp_t, ndim=1] strides
    strides = np.array(strides_from(pmask_shape, np.bool), dtype=np.intp)

    # First do the interior contributions.
    # We first figure out which vertices, edges, triangles, tetrahedra
    # are uniquely associated with an interior voxel
    union = join_complexes(*[cube_with_strides_center((0,0,1), strides),
                             cube_with_strides_center((0,1,0), strides),
                             cube_with_strides_center((0,1,1), strides),
                             cube_with_strides_center((1,0,0), strides),
                             cube_with_strides_center((1,0,1), strides),
                             cube_with_strides_center((1,1,0), strides),
                             cube_with_strides_center((1,1,1), strides)])
    c = cube_with_strides_center((0,0,0), strides)

    d4 = np.array(list(c[4].difference(union[4])))
    d3 = np.array(list(c[3].difference(union[3])))
    d2 = np.array(list(c[2].difference(union[2])))

    ds2 = d2.shape[0]
    ds3 = d3.shape[0]
    ds4 = d4.shape[0]

    ss0 = strides[0]
    ss1 = strides[1]
    ss2 = strides[2]

    nvox = mask.size

    for i in range(s0-1):
        for j in range(s1-1):
            for k in range(s2-1):
                index = i*ss0+j*ss1+k*ss2
                for l in range(ds4):
                    v0 = index + d4[l,0]
                    m = fpmask[v0]
                    if m:
                        v1 = index + d4[l,1]
                        v2 = index + d4[l,2]
                        v3 = index + d4[l,3]
                        m = m * fpmask[v1] * fpmask[v2] * fpmask[v3]
                        l0 = l0 - m

                for l in range(ds3):
                    v0 = index + d3[l,0]
                    m = fpmask[v0]
                    if m:
                        v1 = index + d3[l,1]
                        v2 = index + d3[l,2]
                        m = m * fpmask[v1] * fpmask[v2]
                        l0 = l0 + m

                for l in range(ds2):
                    v0 = index + d2[l,0]
                    m = fpmask[v0]
                    if m:
                        v1 = index + d2[l,1]
                        m = m * fpmask[v1]
                        l0 = l0 - m
    l0 += mask.sum()
    return l0


def Lips3d(coords, mask):
    """ Estimated intrinsic volumes within masked region given coordinates

    Given a 3d `mask` and coordinates `coords`, estimate the intrinsic volumes
    of the masked region. The region is broken up into tetrahedra / triangles /
    edges / vertices, which are included based on whether all voxels in the
    tetrahedron / triangle / edge / vertex are in the mask or not.

    Parameters
    ----------
    coords : ndarray((N,i,j,k))
         Coordinates for the voxels in the mask. ``N`` will often be 3 (for 3
         dimensional coordinates, but can be any integer > 0
    mask : ndarray((i,j,k), np.int)
         Binary mask determining whether or not
         a voxel is in the mask.

    Returns
    -------
    mu : ndarray
        Array of intrinsic volumes [mu0, mu1, mu2, mu3], being, respectively:
        #. Euler characteristic
        #. 2 * mean caliper diameter
        #. 0.5 * surface area
        #. Volume.

    Notes
    -----
    The array mask is assumed to be binary. At the time of writing, it
    is not clear how to get cython to use np.bool arrays.

    The 3d cubes are triangulated into 6 tetrahedra of equal volume, as
    described in the reference below.

    References
    ----------
    Taylor, J.E. & Worsley, K.J. (2007). "Detecting sparse signal in random fields,
      with an application to brain mapping."
      Journal of the American Statistical Association, 102(479):913-928.
    """
    if mask.shape != coords.shape[1:]:
        raise ValueError('shape of mask does not match coordinates')
    # if the data can be squeezed, we must use the lower dimensional function
    mask = np.squeeze(mask)
    if mask.ndim < 3:
        value = np.zeros(4)
        coords = coords.reshape((coords.shape[0],) + mask.shape)
        if mask.ndim == 2:
            value[:3] = Lips2d(coords, mask)
        elif mask.ndim == 1:
            value[:2] = Lips1d(coords, mask)
        return value

    if not set(np.unique(mask)).issubset([0,1]):
      raise ValueError('mask should be filled with 0/1 '
                       'values, but be of type np.int')
    cdef:
        # c-level versions of the arrays
        np.ndarray[np.float_t, ndim=4] coords_c
        np.ndarray[np.intp_t, ndim=3] mask_c
        # 'flattened' coords (2d array)
        np.ndarray[np.float_t, ndim=2] fcoords
        np.ndarray[np.float_t, ndim=2] D
        # 'flattened' mask (1d array)
        np.ndarray[np.intp_t, ndim=1] fmask
        np.ndarray[np.intp_t, ndim=1] fpmask
        np.ndarray[np.intp_t, ndim=3] pmask
        # d3 and d4 are lists of triangles and tetrahedra
        # associated to particular voxels in the cube
        np.ndarray[np.intp_t, ndim=2] d4
        np.ndarray[np.intp_t, ndim=2] m4
        np.ndarray[np.intp_t, ndim=2] d3
        np.ndarray[np.intp_t, ndim=2] m3
        np.ndarray[np.intp_t, ndim=2] d2
        np.ndarray[np.intp_t, ndim=2] m2
        np.ndarray[np.intp_t, ndim=1] cvertices
        # scalars
        np.npy_intp i, j, k, l, s0, s1, s2, ds4, ds3, ds2
        np.npy_intp index, pindex, m, nvox, r, s, rr, ss, mr, ms
        np.npy_intp ss0, ss1, ss2 # strides
        np.npy_intp v0, v1, v2, v3 # vertices for mask
        np.npy_intp w0, w1, w2, w3 # vertices for data
        double l0, l1, l2, l3
        double res

    coords_c = coords
    mask_c = mask
    l0 = 0; l1 = 0; l2 = 0; l3 = 0

    pmask_shape = np.array(mask.shape) + 1
    pmask = np.zeros(pmask_shape, np.int)
    pmask[:-1,:-1,:-1] = mask_c

    s0, s1, s2 = (pmask.shape[0], pmask.shape[1], pmask.shape[2])

    fpmask = pmask.reshape(-1)
    fmask = mask_c.reshape(-1)
    fcoords = coords_c.reshape((coords_c.shape[0], -1))

    # First do the interior contributions.
    # We first figure out which vertices, edges, triangles, tetrahedra
    # are uniquely associated with an interior voxel

    # The mask is copied into a larger array, hence it will have different
    # strides than the data
    cdef:
        np.ndarray[np.intp_t, ndim=1] strides
        np.ndarray[np.intp_t, ndim=1] dstrides
    strides = np.array(strides_from(pmask_shape, np.bool), dtype=np.intp)
    dstrides = np.array(strides_from(mask.shape, np.bool), dtype=np.intp)
    ss0, ss1, ss2 = strides[0], strides[1], strides[2]
    ss0d, ss1d, ss2d = dstrides[0], dstrides[1], dstrides[2]
    verts = []
    for i in range(2):
        for j in range(2):
            for k in range(2):
                verts.append(ss0d * i + ss1d * j + ss2d * k)
    cvertices = np.array(sorted(verts), np.intp)

    union = join_complexes(*[cube_with_strides_center((0,0,1), strides),
                             cube_with_strides_center((0,1,0), strides),
                             cube_with_strides_center((0,1,1), strides),
                             cube_with_strides_center((1,0,0), strides),
                             cube_with_strides_center((1,0,1), strides),
                             cube_with_strides_center((1,1,0), strides),
                             cube_with_strides_center((1,1,1), strides)])
    c = cube_with_strides_center((0,0,0), strides)
    m4 = np.array(list(c[4].difference(union[4])))
    m3 = np.array(list(c[3].difference(union[3])))
    m2 = np.array(list(c[2].difference(union[2])))

    d4 = np.array([[_convert_stride3(v, strides, (4,2,1)) for v in m4[i]] for i in range(m4.shape[0])])
    d4 = np.hstack([m4, d4])
    ds4 = d4.shape[0]

    d3 = np.array([[_convert_stride3(v, strides, (4,2,1)) for v in m3[i]] for i in range(m3.shape[0])])
    d3 = np.hstack([m3, d3])
    ds3 = d3.shape[0]

    d2 = np.array([[_convert_stride3(v, strides, (4,2,1)) for v in m2[i]] for i in range(m2.shape[0])])
    d2 = np.hstack([m2, d2])
    ds2 = d2.shape[0]

    nvox = mask.size

    D = np.zeros((8,8))

    for i in range(s0-1):
        for j in range(s1-1):
            for k in range(s2-1):

                pindex = i*ss0+j*ss1+k*ss2
                index = i*ss0d+j*ss1d+k*ss2d
                for r in range(8):
                    rr = (index+cvertices[r]) % nvox
                    mr = fmask[rr]
                    for s in range(r+1):
                        res = 0
                        ss = (index+cvertices[s]) % nvox
                        ms = fmask[ss]
                        if mr * ms:
                            for l in range(fcoords.shape[0]):
                                res += fcoords[l,ss] * fcoords[l,rr]
                            D[r,s] = res
                            D[s,r] = res
                        else:
                            D[r,s] = 0
                            D[s,r] = 0

                for l in range(ds4):
                    v0 = pindex + d4[l,0]
                    w0 = d4[l,4]
                    m = fpmask[v0]
                    if m:
                        v1 = pindex + d4[l,1]
                        v2 = pindex + d4[l,2]
                        v3 = pindex + d4[l,3]
                        w1 = d4[l,5]
                        w2 = d4[l,6]
                        w3 = d4[l,7]

                        m = m * fpmask[v1] * fpmask[v2] * fpmask[v3]

                        l3 = l3 + m * mu3_tet(D[w0,w0], D[w0,w1], D[w0,w2], 
                                              D[w0,w3], D[w1,w1], D[w1,w2], 
                                              D[w1,w3], D[w2,w2], D[w2,w3],
                                              D[w3,w3])

                        l2 = l2 - m * mu2_tet(D[w0,w0], D[w0,w1], D[w0,w2], 
                                              D[w0,w3], D[w1,w1], D[w1,w2], 
                                              D[w1,w3], D[w2,w2], D[w2,w3],
                                              D[w3,w3])

                        l1 = l1 + m * mu1_tet(D[w0,w0], D[w0,w1], D[w0,w2], 
                                              D[w0,w3], D[w1,w1], D[w1,w2], 
                                              D[w1,w3], D[w2,w2], D[w2,w3],
                                              D[w3,w3])

                        l0 = l0 - m

                for l in range(ds3):
                    v0 = pindex + d3[l,0]
                    w0 = d3[l,3]
                    m = fpmask[v0]
                    if m:
                        v1 = pindex + d3[l,1]
                        v2 = pindex + d3[l,2]
                        w1 = d3[l,4]
                        w2 = d3[l,5]

                        m = m * fpmask[v1] * fpmask[v2] 
                        l2 = l2 + m * mu2_tri(D[w0,w0], D[w0,w1], D[w0,w2], 
                                              D[w1,w1], D[w1,w2], D[w2,w2]) 

                        l1 = l1 - m * mu1_tri(D[w0,w0], D[w0,w1], D[w0,w2], 
                                              D[w1,w1], D[w1,w2], D[w2,w2]) 

                        l0 = l0 + m

                for l in range(ds2):
                    v0 = pindex + d2[l,0]
                    w0 = d2[l,2]
                    m = fpmask[v0]
                    if m:
                        v1 = pindex + d2[l,1]
                        w1 = d2[l,3]
                        m = m * fpmask[v1]
                        l1 = l1 + m * mu1_edge(D[w0,w0], D[w0,w1], D[w1,w1])

                        l0 = l0 - m

    l0 += mask.sum()
    return np.array([l0, l1, l2, l3])


def _convert_stride3(v, stride1, stride2):
    """
    Take a voxel, expressed as in index in stride1 and
    re-express it as an index in stride2
    """
    v0 = v // stride1[0]
    v -= v0 * stride1[0]
    v1 = v // stride1[1]
    v2 = v - v1 * stride1[1]
    return v0*stride2[0] + v1*stride2[1] + v2*stride2[2]


def _convert_stride2(v, stride1, stride2):
    """
    Take a voxel, expressed as in index in stride1 and
    re-express it as an index in stride2
    """
    v0 = v // stride1[0]
    v1 = v - v0 * stride1[0]
    return v0*stride2[0] + v1*stride2[1]


def _convert_stride1(v, stride1, stride2):
    """
    Take a voxel, expressed as in index in stride1 and
    re-express it as an index in stride2
    """
    v0 = v // stride1[0]
    return v0 * stride2[0]


def Lips2d(coords, mask):
    """ Estimate intrinsic volumes for 2d region in `mask` given `coords`

    Given a 2d `mask` and coordinates `coords`, estimate the intrinsic volumes
    of the masked region. The region is broken up into triangles / edges /
    vertices, which are included based on whether all voxels in the triangle /
    edge / vertex are in the mask or not.

    Parameters
    ----------
    coords : ndarray((N,i,j,k))
         Coordinates for the voxels in the mask. ``N`` will often be 2 (for 2
         dimensional coordinates, but can be any integer > 0
    mask : ndarray((i,j), np.int)
         Binary mask determining whether or not a voxel is in the mask.

    Returns
    -------
    mu : ndarray
        Array of intrinsic volumes [mu0, mu1, mu2], being, respectively:
        #. Euler characteristic
        #. 2 * mean caliper diameter
        #. Area.

    Notes
    -----
    The array mask is assumed to be binary. At the time of writing, it
    is not clear how to get cython to use np.bool arrays.

    References
    ----------
    Taylor, J.E. & Worsley, K.J. (2007). "Detecting sparse signal in random fields,
      with an application to brain mapping."
      Journal of the American Statistical Association, 102(479):913-928.
    """
    if mask.shape != coords.shape[1:]:
        raise ValueError('shape of mask does not match coordinates')
    # if the data can be squeezed, we must use the lower dimensional function
    mask = np.squeeze(mask)
    if mask.ndim == 1:
        value = np.zeros(3)
        coords = coords.reshape((coords.shape[0],) + mask.shape)
        value[:2] = Lips1d(coords, mask)
        return value

    if not set(np.unique(mask)).issubset([0,1]):
      raise ValueError('mask should be filled with 0/1 '
                       'values, but be of type np.int')

    cdef:
        # c-level versions of the arrays
        np.ndarray[np.float_t, ndim=3] coords_c
        np.ndarray[np.intp_t, ndim=2] mask_c
        # 'flattened' coords (2d array)
        np.ndarray[np.float_t, ndim=2] fcoords
        np.ndarray[np.float_t, ndim=2] D
        # 'flattened' mask (1d array)
        np.ndarray[np.intp_t, ndim=1] fmask
        np.ndarray[np.intp_t, ndim=1] fpmask
        np.ndarray[np.intp_t, ndim=2] pmask
        # d2 and d3 are lists of triangles associated to particular voxels in
        # the square
        np.ndarray[np.intp_t, ndim=2] d3
        np.ndarray[np.intp_t, ndim=2] d2
        np.ndarray[np.intp_t, ndim=1] cvertices
        # scalars
        np.npy_intp i, j, k, l, r, s, rr, ss, mr, ms, s0, s1
        np.npy_intp ds2, ds3, index, m, npix
        np.npy_intp ss0, ss1, ss0d, ss1d # strides
        np.npy_intp v0, v1, v2 # vertices
        double l0, l1, l2
        double res

    coords_c = coords
    mask_c = mask
    l0 = 0; l1 = 0; l2 = 0

    pmask_shape = np.array(mask.shape) + 1
    pmask = np.zeros(pmask_shape, np.int)
    pmask[:-1,:-1] = mask_c

    s0, s1 = pmask.shape[0], pmask.shape[1]

    fpmask = pmask.reshape(-1)
    fmask = mask_c.reshape(-1)
    fcoords = coords.reshape((coords.shape[0], -1))

    # First do the interior contributions.
    # We first figure out which vertices, edges, triangles, tetrahedra
    # are uniquely associated with an interior voxel

    # The mask is copied into a larger array, hence it will have different
    # strides than the data
    cdef:
        np.ndarray[np.intp_t, ndim=1] strides
        np.ndarray[np.intp_t, ndim=1] dstrides
    strides = np.array(strides_from(pmask_shape, np.bool), dtype=np.intp)
    dstrides = np.array(strides_from(mask.shape, np.bool), dtype=np.intp)
    ss0, ss1 = strides[0], strides[1]
    ss0d, ss1d = dstrides[0], dstrides[1]
    verts = []
    for i in range(2):
        for j in range(2):
            verts.append(ss0d * i + ss1d * j)
    cvertices = np.array(sorted(verts), np.intp)

    union = join_complexes(*[cube_with_strides_center((0,1), strides),
                             cube_with_strides_center((1,0), strides),
                             cube_with_strides_center((1,1), strides)])

    c = cube_with_strides_center((0,0), strides)
    m3 = np.array(list(c[3].difference(union[3])))
    m2 = np.array(list(c[2].difference(union[2])))

    d3 = np.array([[_convert_stride2(v, strides, (2,1)) for v in m3[i]] for i in range(m3.shape[0])])
    d3 = np.hstack([m3, d3])
    ds3 = d3.shape[0]

    d2 = np.array([[_convert_stride2(v, strides, (2,1)) for v in m2[i]] for i in range(m2.shape[0])])
    d2 = np.hstack([m2, d2])
    ds2 = d2.shape[0]

    D = np.zeros((4,4))

    npix = mask.size

    for i in range(s0-1):
        for j in range(s1-1):
            pindex = i*ss0+j*ss1
            index = i*ss0d+j*ss1d
            for r in range(4):
                rr = (index+cvertices[r]) % npix
                mr = fmask[rr]
                for s in range(r+1):
                    res = 0
                    ss = (index+cvertices[s]) % npix
                    ms = fmask[ss]
                    if mr * ms:
                        for l in range(fcoords.shape[0]):
                            res += fcoords[l,ss] * fcoords[l,rr]
                            D[r,s] = res
                            D[s,r] = res
                    else:
                        D[r,s] = 0
                        D[s,r] = 0

            for l in range(ds3):
                v0 = pindex + d3[l,0]
                w0 = d3[l,3]
                m = fpmask[v0]
                if m:
                    v1 = pindex + d3[l,1]
                    v2 = pindex + d3[l,2]
                    w1 = d3[l,4]
                    w2 = d3[l,5]
                    m = m * fpmask[v1] * fpmask[v2]
                    l2 = l2 + mu2_tri(D[w0,w0], D[w0,w1], D[w0,w2],
                                        D[w1,w1], D[w1,w2], D[w2,w2]) * m
                    l1 = l1 - mu1_tri(D[w0,w0], D[w0,w1], D[w0,w2],
                                        D[w1,w1], D[w1,w2], D[w2,w2]) * m
                    l0 = l0 + m

            for l in range(ds2):
                v0 = pindex + d2[l,0]
                w0 = d2[l,2]
                m = fpmask[v0]
                if m:
                    v1 = pindex + d2[l,1]
                    w1 = d2[l,3]
                    m = m * fpmask[v1]
                    l1 = l1 + m * mu1_edge(D[w0,w0], D[w0,w1], D[w1,w1])
                    l0 = l0 - m

    l0 += mask.sum()
    return np.array([l0,l1,l2])


def EC2d(mask):
    """ Compute Euler characteristic of 2D region in `mask`

    Given a 2d `mask`, compute the 0th intrinsic volume (Euler characteristic)
    of the masked region. The region is broken up into triangles / edges /
    vertices, which are included based on whether all voxels in the triangle /
    edge / vertex are in the mask or not.

    Parameters
    ----------
    mask : ndarray((i,j), np.int)
         Binary mask determining whether or not a voxel is in the mask.

    Returns
    -------
    mu0 : int
        Euler characteristic

    Notes
    -----
    The array mask is assumed to be binary. At the time of writing, it
    is not clear how to get cython to use np.bool arrays.

    References
    ----------
    Taylor, J.E. & Worsley, K.J. (2007). "Detecting sparse signal in random fields,
      with an application to brain mapping."
      Journal of the American Statistical Association, 102(479):913-928.
    """
    if not set(np.unique(mask)).issubset([0,1]):
      raise ValueError('mask should be filled with 0/1 '
                       'values, but be of type np.int')
    cdef:
        # c-level versions of the array
        np.ndarray[np.intp_t, ndim=2] mask_c
        # 'flattened' mask (1d array)
        np.ndarray[np.intp_t, ndim=1] fpmask
        # d2 and d3 are lists of triangles and tetrahedra
        # associated to particular voxels in the cuve
        np.ndarray[np.intp_t, ndim=2] d2
        np.ndarray[np.intp_t, ndim=2] d3
        # scalars
        np.npy_intp i, j, k, l, s0, s1, ds2, ds3, index, m
        np.npy_intp ss0, ss1 # strides
        np.npy_intp v0, v1 # vertices
        long l0 = 0

    mask_c = mask

    pmask_shape = np.array(mask.shape) + 1
    pmask = np.zeros(pmask_shape, np.int)
    pmask[:-1,:-1] = mask_c

    s0, s1 = (pmask.shape[0], pmask.shape[1])

    fpmask = pmask.reshape(-1)

    cdef:
        np.ndarray[np.intp_t, ndim=1] strides
    strides = np.array(strides_from(pmask_shape, np.bool), dtype=np.intp)
    ss0, ss1 = strides[0], strides[1]

    # First do the interior contributions.
    # We first figure out which vertices, edges, triangles, tetrahedra
    # are uniquely associated with an interior voxel
    union = join_complexes(*[cube_with_strides_center((0,1), strides),
                             cube_with_strides_center((1,0), strides),
                             cube_with_strides_center((1,1), strides)])
    c = cube_with_strides_center((0,0), strides)

    d3 = np.array(list(c[3].difference(union[3])))
    d2 = np.array(list(c[2].difference(union[2])))

    ds2 = d2.shape[0]
    ds3 = d3.shape[0]

    for i in range(s0-1):
        for j in range(s1-1):
            index = i*ss0+j*ss1
            for l in range(ds3):
                v0 = index + d3[l,0]
                m = fpmask[v0]
                if m and v0:
                    v1 = index + d3[l,1]
                    v2 = index + d3[l,2]
                    m = m * fpmask[v1] * fpmask[v2]
                    l0 = l0 + m

            for l in range(ds2):
                v0 = index + d2[l,0]
                m = fpmask[v0]
                if m:
                    v1 = index + d2[l,1]
                    m = m * fpmask[v1]
                    l0 = l0 - m

    l0 += mask.sum()
    return l0


def Lips1d(np.ndarray[np.float_t, ndim=2] coords,
           np.ndarray[np.intp_t, ndim=1] mask):
    """ Estimate intrinsic volumes for 1D region in `mask` given `coords`

    Given a 1d `mask` and coordinates `coords`, estimate the intrinsic volumes
    of the masked region. The region is broken up into edges / vertices, which
    are included based on whether all voxels in the edge / vertex are in the
    mask or not.

    Parameters
    ----------
    coords : ndarray((N,i,j,k))
         Coordinates for the voxels in the mask. ``N`` will often be 1 (for 1
         dimensional coordinates, but can be any integer > 0
    mask : ndarray((i,), np.int)
         Binary mask determining whether or not a voxel is in the mask.

    Returns
    -------
    mu : ndarray
        Array of intrinsic volumes [mu0, mu1], being, respectively:
        #. Euler characteristic
        #. Line segment length

    Notes
    -----
    The array mask is assumed to be binary. At the time of writing, it
    is not clear how to get cython to use np.bool arrays.

    References
    ----------
    Taylor, J.E. & Worsley, K.J. (2007). "Detecting sparse signal in random fields,
      with an application to brain mapping."
      Journal of the American Statistical Association, 102(479):913-928.
    """
    if mask.shape[0] != coords.shape[1]:
        raise ValueError('shape of mask does not match coordinates')
    if not set(np.unique(mask)).issubset([0,1]):
      raise ValueError('mask should be filled with 0/1 '
                       'values, but be of type np.int')
    cdef:
        np.npy_intp i, l, r, s, rr, ss, mr, ms, s0, index, m
        double l0, l1
        double res

    l0 = 0; l1 = 0
    s0 = mask.shape[0]
    D = np.zeros((2,2))

    for i in range(s0):
        for r in range(2):
            rr = (i+r) % s0
            mr = mask[rr]
            for s in range(r+1):
                res = 0
                ss = (i+s) % s0
                ms = mask[ss]
                if mr * ms * ((i+r) < s0) * ((i+s) < s0):
                    for l in range(coords.shape[0]):
                        res += coords[l,ss] * coords[l,rr]
                        D[r,s] = res
                        D[s,r] = res
                else:
                    D[r,s] = 0
                    D[s,r] = 0

        m = mask[i]
        if m:
            m = m * (mask[(i+1) % s0] * ((i+1) < s0))
            l1 = l1 + m * mu1_edge(D[0,0], D[0,1], D[1,1])
            l0 = l0 - m

    l0 += mask.sum()
    return np.array([l0,l1])


def EC1d(np.ndarray[np.intp_t, ndim=1] mask):
    """ Compute Euler characteristic for 1d `mask`

    Given a 1d mask `mask`, compute the 0th intrinsic volume (Euler
    characteristic) of the masked region. The region is broken up into edges /
    vertices, which are included based on whether all voxels in the edge /
    vertex are in the mask or not.

    Parameters
    ----------
    mask : ndarray((i,), np.int)
         Binary mask determining whether or not a voxel is in the mask.

    Returns
    -------
    mu0 : int
        Euler characteristic

    Notes
    -----
    The array mask is assumed to be binary. At the time of writing, it
    is not clear how to get cython to use np.bool arrays.

    The 3d cubes are triangulated into 6 tetrahedra of equal volume, as
    described in the reference below.

    References
    ----------
    Taylor, J.E. & Worsley, K.J. (2007). "Detecting sparse signal in random fields,
      with an application to brain mapping."
      Journal of the American Statistical Association, 102(479):913-928.
    """
    if not set(np.unique(mask)).issubset([0,1]):
      raise ValueError('mask should be filled with 0/1 '
                       'values, but be of type np.int')
    cdef:
        np.npy_intp i, m, s0
        double l0 = 0

    s0 = mask.shape[0]
    for i in range(s0):
        m = mask[i]
        if m:
            m = m * (mask[(i+1) % s0] * ((i+1) < s0))
            l0 = l0 - m

    l0 += mask.sum()
    return l0



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

from ..slices import bounding_box, zslice, yslice, xslice
from ..coordinate_map import AffineTransform
from ..coordinate_system import CoordinateSystem as CS
from ..spaces import scanner_csm, scanner_space, mni_csm

from nose.tools import (
    assert_equal,
    assert_true,
    assert_false)
from numpy.testing import (
    assert_array_equal,
    assert_array_almost_equal)

# Names for a 3D axis set
names = ['xspace', 'yspace', 'zspace']

def test_bounding_box():
    shape = (10, 14, 16)
    coordmap = AffineTransform.identity(names)
    assert_equal(bounding_box(coordmap, shape),
                 ((0., 9.), (0, 13), (0, 15)))


def test_box_slice():
    t = xslice(5, ([0, 9], 10), ([0, 9], 10), scanner_space)
    assert_array_almost_equal(t.affine,
                              [[ 0.,  0.,  5.],
                               [ 1.,  0.,  0.],
                               [ 0.,  1.,  0.],
                               [ 0.,  0.,  1.]])
    assert_equal(t.function_domain, CS(['i_y', 'i_z'], 'slice'))
    assert_equal(t.function_range, scanner_csm(3))
    t = yslice(4, ([0, 9], 10), ([0, 9], 10), 'mni')
    assert_array_almost_equal(t.affine,
                              [[ 1.,  0.,  0.],
                               [ 0.,  0.,  4.],
                               [ 0.,  1.,  0.],
                               [ 0.,  0.,  1.]])
    assert_equal(t.function_domain, CS(['i_x', 'i_z'], 'slice'))
    assert_equal(t.function_range, mni_csm(3))
    t = zslice(3, ([0, 9], 10), ([0, 9], 10), mni_csm(3))
    assert_array_almost_equal(t.affine,
                              [[ 1.,  0.,  0.],
                               [ 0.,  1.,  0.],
                               [ 0.,  0.,  3.],
                               [ 0.,  0.,  1.]])
    assert_equal(t.function_domain, CS(['i_x', 'i_y'], 'slice'))
    assert_equal(t.function_range, mni_csm(3))



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function # Python 2/3 compatibility
__doc__ = """
Example of a demo that fits a Bayesian Gaussian Mixture Model (GMM)
to a dataset.

Variational bayes and Gibbs estimation are sucessively run on the same
dataset.

Requires matplotlib

Author : Bertrand Thirion, 2008-2010
"""
print(__doc__)

import numpy as np
import numpy.random as nr

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise RuntimeError("This script needs the matplotlib library")

import nipy.algorithms.clustering.bgmm as bgmm
from nipy.algorithms.clustering.gmm import plot2D

dim = 2

###############################################################################
# 1. generate a 3-components mixture
x1 = nr.randn(25, dim)
x2 = 3 + 2 * nr.randn(15, dim)
x3 = np.repeat(np.array([-2, 2], ndmin=2), 10, 0) + 0.5 * nr.randn(10, dim)
x = np.concatenate((x1, x2, x3))

###############################################################################
#2. fit the mixture with a bunch of possible models, using Variational Bayes
krange = range(1, 10)
be = - np.inf
for  k in krange:
    b = bgmm.VBGMM(k, dim)
    b.guess_priors(x)
    b.initialize(x)
    b.estimate(x)
    ek = float(b.evidence(x))
    if ek > be:
        be = ek
        bestb = b
    print(k, 'classes, free energy:', ek)

###############################################################################
# 3. plot the result
z = bestb.map_label(x)
plot2D(x, bestb, z, verbose=0)
plt.title('Variational Bayes')

###############################################################################
# 4. the same, with the Gibbs GMM algo
niter = 1000
krange = range(1, 6)
bbf = - np.inf
for k in krange:
    b = bgmm.BGMM(k, dim)
    b.guess_priors(x)
    b.initialize(x)
    b.sample(x, 100)
    w, cent, prec, pz = b.sample(x, niter=niter, mem=1)
    bplugin = bgmm.BGMM(k, dim, cent, prec, w)
    bplugin.guess_priors(x)
    bfk = bplugin.bayes_factor(x, pz.astype(np.int), nperm=120)
    print(k, 'classes, evidence:', bfk)
    if bfk > bbf:
        bestk = k
        bbf = bfk
        bbgmm = bplugin

z = bbgmm.map_label(x)
plot2D(x, bbgmm, z, verbose=0)
plt.title('Gibbs sampling')
plt.show()



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
""" General matlab interface code

This is for nipy convenience.  If you're doing heavy matlab interfacing, please
use NiPype instead:

http://nipy.org/nipype
"""

# Stdlib imports
import os
import re
import tempfile
import subprocess


matlab_cmd = 'matlab -nojvm -nosplash'

def run_matlab(cmd):
    subprocess.call('%s -r \"%s;exit\" ' % (matlab_cmd, cmd),
                    shell=True)


def run_matlab_script(script_lines, script_name='pyscript'):
    ''' Put multiline matlab script into script file and run '''
    mfile = open(script_name + '.m', 'wt')
    mfile.write(script_lines)
    mfile.close()
    return run_matlab(script_name)


# Functions, classes and other top-level code
def mlab_tempfile(dir=None):
    """Returns a temporary file-like object with valid matlab name.

    The file name is accessible as the .name attribute of the returned object.
    The caller is responsible for closing the returned object, at which time
    the underlying file gets deleted from the filesystem.

    Parameters
    ----------
    dir : str
        A path to use as the starting directory.  Note that this directory must
        already exist, it is NOT created if it doesn't (in that case, OSError is
        raised instead).

    Returns
    -------
    f : file-like object

    Examples
    --------
    >>> f = mlab_tempfile()
    >>> pth, fname = os.path.split(f.name)
    >>> '-' not in fname
    True
    >>> f.close()
    """
    valid_name = re.compile(r'^\w+$')
    # Make temp files until we get one whose name is a valid matlab identifier,
    # since matlab imposes that constraint.  Since the temp file routines may
    # return names that aren't valid matlab names, but we can't control that
    # directly, we just keep trying until we get a valid name.  To avoid an
    # infinite loop for some strange reason, we only try 100 times.
    for n in range(100):
        f = tempfile.NamedTemporaryFile(suffix='.m',prefix='tmp_matlab_',
                                        dir=dir)
        # Check the file name for matlab compliance
        fname =  os.path.splitext(os.path.basename(f.name))[0]
        if valid_name.match(fname):
            break
        # Close the temp file we just made if its name is not valid; the
        # tempfile module then takes care of deleting the actual file on disk.
        f.close()
    else:
        raise ValueError("Could not make temp file after 100 tries")
    return f



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function # Python 2/3 compatibility
__doc__ = """
This example simulates a number of pure Gaussian white noise signals, then fits
each one in terms of two regressors: a constant baseline, and a linear function
of time. The voxelwise t statistics associated with the baseline coefficient are
then computed.
"""
print(__doc__)

import numpy as np

from nipy.modalities.fmri.glm import GeneralLinearModel

dimt = 100
dimx = 10
dimy = 11
dimz = 12

# axis defines the "time direction"
y = np.random.randn(dimt, dimx * dimy * dimz)
axis = 0

X = np.array([np.ones(dimt), range(dimt)])
X = X.T ## the design matrix X must have dimt lines

mod = GeneralLinearModel(X)
mod.fit(y)

# Define a t contrast
tcon = mod.contrast([1, 0])

# Compute the t-stat
t = tcon.stat()
## t = tcon.stat(baseline=1) to test effects > 1

# Compute the p-value
p = tcon.p_value()

# Compute the z-score
z = tcon.z_score()

# Perform a F test without keeping the F stat
p = mod.contrast([[1, 0], [1, - 1]]).p_value()

print(np.shape(y))
print(np.shape(X))
print(np.shape(z))



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Test the mask-extracting utilities.
"""

from __future__ import with_statement

import numpy as np

import nibabel as nib
from nibabel.tmpdirs import InTemporaryDirectory

from .. import mask as nnm
from ..mask import largest_cc, threshold_connect_components, \
        series_from_mask

from nipy.testing import assert_equal, assert_true, \
    assert_array_equal, anatfile, assert_false


def test_largest_cc():
    """ Check the extraction of the largest connected component.
    """
    a = np.zeros((6, 6, 6))
    a[1:3, 1:3, 1:3] = 1
    yield assert_equal, a, largest_cc(a)
    b = a.copy()
    b[5, 5, 5] = 1
    yield assert_equal, a, largest_cc(b)


def test_threshold_connect_components():
    a = np.zeros((10, 10))
    a[0, 0] = 1
    a[3, 4] = 1
    a = threshold_connect_components(a, 2)
    yield assert_true, np.all(a == 0)
    a[0, 0:3] = 1
    b = threshold_connect_components(a, 2)
    yield assert_true, np.all(a == b)


def test_mask():
    mean_image = np.ones((9, 9))
    mean_image[3:-3, 3:-3] = 10
    mean_image[5, 5] = 100
    mask1 = nnm.compute_mask(mean_image)
    mask2 = nnm.compute_mask(mean_image, exclude_zeros=True)
    # With an array with no zeros, exclude_zeros should not make
    # any difference
    yield assert_array_equal, mask1, mask2
    # Check that padding with zeros does not change the extracted mask
    mean_image2 = np.zeros((30, 30))
    mean_image2[:9, :9] = mean_image
    mask3 = nnm.compute_mask(mean_image2, exclude_zeros=True)
    yield assert_array_equal, mask1, mask3[:9, :9]
    # However, without exclude_zeros, it does
    mask3 = nnm.compute_mask(mean_image2)
    yield assert_false, np.allclose(mask1, mask3[:9, :9])
    # check that  opening is 2 by default
    mask4 = nnm.compute_mask(mean_image, exclude_zeros=True, opening=2)
    yield assert_array_equal, mask1, mask4
    # check that opening has an effect
    mask5 = nnm.compute_mask(mean_image, exclude_zeros=True, opening=0)
    yield assert_true, mask5.sum() > mask4.sum()
    

def test_mask_files():
    with InTemporaryDirectory():
        # Make a 4D file from the anatomical example
        img = nib.load(anatfile)
        arr = img.get_data()
        a2 = np.zeros(arr.shape + (2, ))
        a2[:, :, :, 0] = arr
        a2[:, :, :, 1] = arr
        img = nib.Nifti1Image(a2, np.eye(4))
        a_fname = 'fourd_anat.nii'
        nib.save(img, a_fname)
        # check 4D mask
        msk1, mean1 = nnm.compute_mask_files(a_fname, return_mean=True)
        # and mask from identical list of 3D files
        msk2, mean2 = nnm.compute_mask_files([anatfile, anatfile],
                                             return_mean=True)
        yield assert_array_equal, msk1, msk2
        yield assert_array_equal, mean1, mean2


def test_series_from_mask():
    """ Test the smoothing of the timeseries extraction
    """
    # A delta in 3D
    data = np.zeros((40, 40, 40, 2))
    data[20, 20, 20] = 1
    mask = np.ones((40, 40, 40), dtype=np.bool)
    with InTemporaryDirectory():
        for affine in (np.eye(4), np.diag((1, 1, -1, 1)),
                        np.diag((.5, 1, .5, 1))):
            img = nib.Nifti1Image(data, affine)
            nib.save(img, 'testing.nii')
            series, header = series_from_mask('testing.nii', mask, smooth=9)
            series = np.reshape(series[:, 0], (40, 40, 40))
            vmax = series.max()
            # We are expecting a full-width at half maximum of
            # 9mm/voxel_size:
            above_half_max = series > .5*vmax
            for axis in (0, 1, 2):
                proj = np.any(np.any(np.rollaxis(above_half_max,
                                axis=axis), axis=-1), axis=-1)
                assert_equal(proj.sum(), 9/np.abs(affine[axis, axis]))

        # Check that NaNs in the data do not propagate
        data[10, 10, 10] = np.NaN
        img = nib.Nifti1Image(data, affine)
        nib.save(img, 'testing.nii')
        series, header = series_from_mask('testing.nii', mask, smooth=9)
        assert_true(np.all(np.isfinite(series)))

def test_compute_mask_sessions():
    """Test that the mask computes well on multiple sessions
    """
    with InTemporaryDirectory():
        # Make a 4D file from the anatomical example
        img = nib.load(anatfile)
        arr = img.get_data()
        a2 = np.zeros(arr.shape + (2, ))
        a2[:, :, :, 0] = arr
        a2[:, :, :, 1] = arr
        img = nib.Nifti1Image(a2, np.eye(4))
        a_fname = 'fourd_anat.nii'
        nib.save(img, a_fname)
        a3 = a2.copy()
        a3[:10, :10, :10] = 0
        img2 = nib.Nifti1Image(a3, np.eye(4))
        # check 4D mask
        msk1 = nnm.compute_mask_sessions([img2, img2])
        msk2 = nnm.compute_mask_sessions([img2, a_fname])
        assert_array_equal(msk1, msk2)
        msk3 = nnm.compute_mask_sessions([img2, a_fname], threshold=.9)
        msk4 = nnm.compute_mask_sessions([img2, a_fname], threshold=0)
        msk5 = nnm.compute_mask_sessions([a_fname, a_fname])
        assert_array_equal(msk1, msk3)
        assert_array_equal(msk4, msk5)

if __name__ == "__main__":
    import nose
    nose.run(argv=['', __file__])



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import numpy as np

from nibabel import Nifti1Image
from ..utils.simul_multisubject_fmri_dataset import \
    surrogate_2d_dataset
from ..statistical_mapping import cluster_stats

def make_surrogate_data():
    """ Return a single deterministic 3D image 
    """
    shape = (40, 40)
    pos   = np.array([[ 2, 10],
                      [10,  4],
                      [20, 30],
                      [30, 20]])
    ampli = np.array([5, 5, 5, 5])
    data = surrogate_2d_dataset(n_subj=1, pos=pos, shape=shape, noise_level=0,
                                ampli=ampli, spatial_jitter=0,
                                signal_jitter=0).squeeze()
    data = np.reshape(data, (shape[0], shape[1], 1))
    return Nifti1Image(data, np.eye(4))
    
def test1():
    img = make_surrogate_data()
    clusters, info = cluster_stats(img, img, height_th=3., height_control='None', cluster_th=0, nulls={})
    assert len(clusters)==4
    
def test2():
    img = make_surrogate_data()
    clusters, info = cluster_stats(img, img, height_th=3., height_control='None', cluster_th=5, nulls={})
    assert len(clusters)==4

def test3():
    img = make_surrogate_data()
    clusters, info = cluster_stats(img, img, height_th=3., height_control='None', cluster_th=10, nulls={})
    assert len(clusters)==0


def test_4():
    img = make_surrogate_data()
    clusters, info = cluster_stats(img, img, height_th=.001, height_control='fpr', cluster_th=0, nulls={})
    assert len(clusters)==4
    
def test_5():
    img = make_surrogate_data()
    clusters, info = cluster_stats(img, img, height_th=.05, height_control='bonferroni', cluster_th=0, nulls={})
    assert len(clusters)==4

def test_6():
    img = make_surrogate_data()
    clusters, info = cluster_stats(img, img, height_th=.05, 
                                   height_control='fdr', cluster_th=0, nulls={})
    print len(clusters), sum([c['size'] for c in clusters])
    assert len(clusters)==4

def test7():
    img = make_surrogate_data()
    clusters, info = cluster_stats(img, img, height_th=3., height_control='None', cluster_th=0, nulls={})
    nstv = sum([c['size'] for c in clusters])
    assert nstv==36

def test_8():
    img = make_surrogate_data()
    clusters, info = cluster_stats(img, img, height_th=.001, height_control='fpr', cluster_th=0, nulls={})
    nstv = sum([c['size'] for c in clusters])
    assert nstv==36
 
if __name__ == "__main__":
    import nose
    nose.run(argv=['', __file__])




#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function # Python 2/3 compatibility
__doc__ = """
Example of script to parcellate the data from one subject, using various
algorithms.

Note that it can take some time.

author: Bertrand Thirion, 2005-2009
"""
print(__doc__)

from os import mkdir, getcwd, path

from numpy import array

from nipy.labs.spatial_models.parcel_io import fixed_parcellation

# Local import
from get_data_light import DATA_DIR, get_second_level_dataset

# ------------------------------------
# Get the data (mask+functional image)
# take several experimental conditions
# time courses could be used instead

n_beta = [29]
mask_image = path.join(DATA_DIR, 'mask.nii.gz')
betas = [path.join(DATA_DIR, 'spmT_%04d.nii.gz' % n) for n in n_beta]
missing_file = array([not path.exists(m) for m in [mask_image] + betas]).any()
if missing_file:
    get_second_level_dataset()

# set the parameters
n_parcels = 500
mu = 10
nn = 6
verbose = 1
# write directory
write_dir = path.join(getcwd(), 'results')
if not path.exists(write_dir):
    mkdir(write_dir)


lpa = fixed_parcellation(mask_image, betas, n_parcels, nn, 'gkm',
                         write_dir, mu, verbose)
lpa = fixed_parcellation(mask_image, betas, n_parcels, nn, 'ward',
                         write_dir, mu, verbose)
lpa = fixed_parcellation(mask_image, betas, n_parcels, nn, 'ward_and_gkm',
                         write_dir, mu, verbose)



"""
This module is for canonical hrf specification.
Here we provide for SPM, Glover hrfs and finite timpulse response (FIR) models.
This module closely follows SPM implementation

Author: Bertrand Thirion, 2011
"""

import numpy as np
from scipy.stats import gamma


def gamma_difference_hrf(tr, oversampling=16, time_length=32., onset=0.,
                         delay=6, undershoot=16., dispersion=1.,
                         u_dispersion=1., ratio=0.167):
    """ Compute an hrf as the difference of two gamma functions

    Parameters
    ----------
    tr: float, scan repeat time, in seconds
    oversampling: int, temporal oversampling factor, optional
    time_length: float, hrf kernel length, in seconds
    onset: float, onset of the hrf

    Returns
    -------
    hrf: array of shape(length / tr * oversampling, float),
         hrf sampling on the oversampled time grid
    """
    dt = tr / oversampling
    time_stamps = np.linspace(0, time_length, float(time_length) / dt)
    time_stamps -= onset / dt
    hrf = gamma.pdf(time_stamps, delay / dispersion, dt / dispersion) - \
        ratio * gamma.pdf(
        time_stamps, undershoot / u_dispersion, dt / u_dispersion)
    hrf /= hrf.sum()
    return hrf


def spm_hrf(tr, oversampling=16, time_length=32., onset=0.):
    """ Implementation of the SPM hrf model

    Parameters
    ----------
    tr: float, scan repeat time, in seconds
    oversampling: int, temporal oversampling factor, optional
    time_length: float, hrf kernel length, in seconds
    onset: float, onset of the response

    Returns
    -------
    hrf: array of shape(length / tr * oversampling, float),
         hrf sampling on the oversampled time grid
    """
    return gamma_difference_hrf(tr, oversampling, time_length, onset)


def glover_hrf(tr, oversampling=16, time_length=32., onset=0.):
    """ Implementation of the Glover hrf model

    Parameters
    ----------
    tr: float, scan repeat time, in seconds
    oversampling: int, temporal oversampling factor, optional
    time_length: float, hrf kernel length, in seconds
    onset: float, onset of the response

    Returns
    -------
    hrf: array of shape(length / tr * oversampling, float),
         hrf sampling on the oversampled time grid
    """
    return gamma_difference_hrf(tr, oversampling, time_length, onset,
                                delay=6, undershoot=12., dispersion=.9,
                                u_dispersion=.9, ratio=.35)


def spm_time_derivative(tr, oversampling=16, time_length=32., onset=0.):
    """Implementation of the SPM time derivative hrf (dhrf) model

    Parameters
    ----------
    tr: float, scan repeat time, in seconds
    oversampling: int, temporal oversampling factor, optional
    time_length: float, hrf kernel length, in seconds
    onset: float, onset of the response

    Returns
    -------
    dhrf: array of shape(length / tr, float),
          dhrf sampling on the provided grid
    """
    do = .1
    dhrf = 1. / do * (spm_hrf(tr, oversampling, time_length, onset + do) -
                      spm_hrf(tr, oversampling, time_length, onset))
    return dhrf


def glover_time_derivative(tr, oversampling=16, time_length=32., onset=0.):
    """Implementation of the flover time derivative hrf (dhrf) model

    Parameters
    ----------
    tr: float, scan repeat time, in seconds
    oversampling: int, temporal oversampling factor, optional
    time_length: float, hrf kernel length, in seconds
    onset: float, onset of the response

    Returns
    -------
    dhrf: array of shape(length / tr, float),
          dhrf sampling on the provided grid
    """
    do = .1
    dhrf = 1. / do * (glover_hrf(tr, oversampling, time_length, onset + do) -
                      glover_hrf(tr, oversampling, time_length, onset))
    return dhrf


def spm_dispersion_derivative(tr, oversampling=16, time_length=32., onset=0.):
    """Implementation of the SPM dispersion derivative hrf model

    Parameters
    ----------
    tr: float, scan repeat time, in seconds
    oversampling: int, temporal oversampling factor, optional
    time_length: float, hrf kernel length, in seconds
    onset: float, onset of the response

    Returns
    -------
    dhrf: array of shape(length / tr * oversampling, float),
          dhrf sampling on the oversampled time grid
    """
    dd = .01
    dhrf = 1. / dd * (gamma_difference_hrf(tr, oversampling, time_length,
                                           onset, dispersion=1. + dd) -
                      spm_hrf(tr, oversampling, time_length, onset))
    return dhrf


def sample_condition(exp_condition, frametimes, oversampling=16):
    """ this function samples the experimental condition at frametimes

    Parameters
    ----------
    exp_condition: a tuple of 3 arrays of shape n, corresponding
                   to  (onsets, duration, value),
                   describing the experimental condition
    frametimes: array of shape(n)
    over_sampling: int, over_sampling factor

    Returns
    -------
    regressor: array of shape(n)
    """
    # generate the oversampled frame times
    n = frametimes.size
    if oversampling == 1:
        hr_frametimes = frametimes
    else:
        hr_frametimes = np.linspace(0, frametimes.max() * (1 + 1. / (n - 1)),
                                    n * oversampling + 1)

    # get the regressor information
    onsets, duration, values = exp_condition
    onsets, duration, values = np.asanyarray(onsets), np.asanyarray(duration),\
        np.asanyarray(values)

    # generate the regressor time course
    tmax = len(hr_frametimes)
    regressor = np.zeros_like(hr_frametimes).astype(np.float)
    t_onset = np.minimum(np.searchsorted(hr_frametimes, onsets), tmax - 1)
    regressor[t_onset] += values
    t_offset = np.minimum(np.searchsorted(hr_frametimes, onsets + duration),
                          tmax - 1)
    # for event related, shift the offset by 1
    for i, to in enumerate(t_offset):
        if to > 0 and to < tmax - 1 and to == t_onset[i]:
            t_offset[i] += 1

    regressor[t_offset] -= values
    regressor = np.cumsum(regressor)

    # normalize the regressor in case of  block design
    #if (duration > 0).any():
    #    regressor /= oversampling
    return regressor, hr_frametimes


def resample_regressor(hr_regressor, hr_frametimes, frametimes, kind='linear'):
    """ this function samples the regressors at frametimes

    Parameters
    ----------
    hr_regressor: array of shape(n),
                  the regressor time course sampled at high temporal resolution
    hr_frametimes: array of shape(n),
                   the corresponding time stamps
    frametimes: array of shape(p),
                the desired time stamps
    kind: string, optional, the kind of desired interpolation

    Returns
    -------
    regressor: array of shape(p), the resampled regressor
    """
    from scipy.interpolate import interp1d
    f = interp1d(hr_frametimes, hr_regressor)
    return f(frametimes).T


def _orthogonalize(X):
    """ This function orthogonalizes the columns of the design matrix wrt
    to the first column

    Parameters
    ----------
    X: array of shape(n, p), the data to be orthogonalized

    Returns
    -------
    X: after orthogonalization

    Notes
    -----
    X is changed in place. the columns are not normalized
    """
    if X.size == X.shape[0]:
        return X
    from numpy.linalg import pinv
    for i in range(1, X.shape[1]):
        X[:, i] -= np.dot(X[:, i], np.dot(X[:, :i], pinv(X[:, :i])))
    return X


def _regressor_names(con_name, hrf_model, fir_delays=None):
    """ returns a list of regressor names, computed from con-name and hrf type

    Parameters
    ----------
    con_name: string, identifier of the condition
    hrf_model: string, identifier of the hrf model

    Returns
    -------
    names: a list of strings yielding the regressor names
    """
    if hrf_model == 'canonical':
        return [con_name]
    elif hrf_model == "canonical with derivative":
        return [con_name, con_name + "_derivative"]
    elif hrf_model == 'spm':
        return [con_name]
    elif hrf_model == 'spm_time':
        return [con_name, con_name + "_derivative"]
    elif hrf_model == 'spm_time_dispersion':
        return [con_name, con_name + "_derivative", con_name + "_dispersion"]
    elif hrf_model == 'fir':
        return [con_name + "_delay_%d" % i for i in fir_delays]


def _hrf_kernel(hrf_model, tr, oversampling=16, fir_delays=None):
    """ Given the specification of the hemodynamic model and time parameters,
    return the list of matching kernels

    Parameters
    ----------
    hrf_model: string, identifier of the hrf model
    tr: the repetition time in seconds
    oversampling: int, temporal oversampling factor to have a smooth hrf
    fir_delays: list of for delays

    Returns
    -------
    hkernel: a list of hrf kernels, depending on the specified hrf model
    """
    if hrf_model == 'spm':
        hkernel = [spm_hrf(tr, oversampling)]
    elif hrf_model == 'spm_time':
        hkernel = [spm_hrf(tr, oversampling),
                   spm_time_derivative(tr, oversampling)]
    elif hrf_model == 'spm_time_dispersion':
        hkernel = [spm_hrf(tr, oversampling),
                   spm_time_derivative(tr, oversampling),
                   spm_dispersion_derivative(tr, oversampling)]
    elif hrf_model == 'canonical':
        hkernel = [glover_hrf(tr, oversampling)]
    elif hrf_model == 'canonical with derivative':
        hkernel = [glover_hrf(tr, oversampling),
                   glover_time_derivative(tr, oversampling)]
    elif hrf_model == 'fir':
        hkernel = [np.hstack((np.zeros(f * oversampling),
                              np.ones(oversampling)))
                   for f in fir_delays]
    else:
        raise ValueError('Unknown hrf model')
    return hkernel


def compute_regressor(exp_condition, hrf_model, frametimes, con_id='cond',
                      oversampling=16, fir_delays=None):
    """ This is the main function to convolve regressors with hrf model

    Parameters
    ----------
    exp_condition: descriptor of an experimental condition
    hrf_model: string, the hrf model to be used. Can be chosen among:
               'spm', 'spm_time', 'spm_time_dispersion', 'canonical',
               'canonical_derivative', 'fir'
    frametimes: array of shape (n):the sought
    con_id: string, optional identifier of the condition
    oversampling: int, optional, oversampling factor to perform the convolution
    fir_delays: array-like of int, onsets corresponding to the fir basis

    Returns
    -------
    creg: array of shape(n_scans, n_reg): computed regressors sampled
                                          at frametimes
    reg_names: list of strings, corresponding regressor names

    Notes
    -----
    The different hemodynamic models can be understood as follows:
    'spm': this is the hrf model used in spm
    'spm_time': this is the spm model plus its time derivative (2 regressors)
    'spm_time_dispersion': idem, plus dispersion derivative (3 regressors)
    'canonical': this one corresponds to the Glover hrf
    'canonical_derivative': the Glover hrf + time derivative (2 regressors)
    'fir': finite impulse response basis, a set of delayed dirac models
           with arbitrary length. This one currently assumes regularly spaced
           frametimes (i.e. fixed time of repetition).
    It is expected that spm standard and Glover model would not yield
    large differences in most cases.
    """
    # this is the average tr in this session, not necessarily the true tr
    tr = float(frametimes.max()) / (np.size(frametimes) - 1)

    # 1. create the high temporal resolution regressor
    hr_regressor, hr_frametimes = sample_condition(
        exp_condition, frametimes, oversampling)

    # 2. create the  hrf model(s)
    hkernel = _hrf_kernel(hrf_model, tr, oversampling, fir_delays)

    # 3. convolve the regressor and hrf, and downsample the regressor
    conv_reg = np.array([np.convolve(hr_regressor, h)[:hr_regressor.size]
                         for h in hkernel])

    # 4. temporally resample the regressors
    creg = resample_regressor(conv_reg, hr_frametimes, frametimes)

    # 5. ortogonalize the regressors
    if hrf_model != 'fir':
        creg = _orthogonalize(creg)

    # 6 generate regressor names
    reg_names = _regressor_names(con_id, hrf_model, fir_delays=fir_delays)
    return creg, reg_names



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from .resample import resample
from .histogram_registration import (HistogramRegistration, clamp,
                                    ideal_spacing, interp_methods)
from .affine import (threshold, rotation_mat2vec, rotation_vec2mat, to_matrix44,
                     preconditioner, inverse_affine, subgrid_affine, Affine,
                     Affine2D, Rigid, Rigid2D, Similarity, Similarity2D,
                     affine_transforms)
from .groupwise_registration import (interp_slice_order, scanner_coords,
                                     make_grid, Image4d, Realign4dAlgorithm,
                                     resample4d, adjust_subsampling,
                                     single_run_realign4d, realign4d, Realign4d,
                                     FmriRealign4d)

from nipy.testing import Tester
test = Tester().test
bench = Tester().bench



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
""" Image interpolators using ndimage.
"""

import os
import tempfile

import numpy as np

from scipy import ndimage

class ImageInterpolator(object):
    """ Interpolate Image instance at arbitrary points in world space
    
    The resampling is done with scipy.ndimage.
    """
    def __init__(self, image, order=3):
        """
        Parameters
        ----------
        image : Image
           Image to be interpolated
        order : int, optional
           order of spline interpolation as used in scipy.ndimage.
           Default is 3.
        """
        self.image = image
        self.order = order
        self._datafile = None
        self._buildknots()

    def _buildknots(self):
        if self.order > 1:
            data = ndimage.spline_filter(
                np.nan_to_num(self.image.get_data()),
                self.order)
        else:
            data = np.nan_to_num(self.image.get_data())
        if self._datafile is None:
            _, fname = tempfile.mkstemp()
            self._datafile = open(fname, mode='wb')
        else:
            self._datafile = open(self._datafile.name, 'wb')
        data = np.nan_to_num(data.astype(np.float64))
        data.tofile(self._datafile)
        datashape = data.shape
        dtype = data.dtype
        del(data)
        self._datafile.close()
        self._datafile = open(self._datafile.name)
        self.data = np.memmap(self._datafile.name, dtype=dtype,
                              mode='r+', shape=datashape)

    def __del__(self):
        if self._datafile:
            self._datafile.close()
            try:
                os.remove(self._datafile.name)
            except:
                pass

    def evaluate(self, points):
        """ Resample image at points in world space
        
        Parameters
        ----------
        points : array
           values in self.image.coordmap.output_coords.  Each row is a
	   point. 

        Returns
        -------
        V : ndarray
           interpolator of self.image evaluated at points
        """
        points = np.array(points, np.float64)
        output_shape = points.shape[1:]
        points.shape = (points.shape[0], np.product(output_shape))
        cmapi = self.image.coordmap.inverse()
        voxels = cmapi(points.T).T
        V = ndimage.map_coordinates(self.data, 
                                     voxels,
                                     order=self.order,
                                     prefilter=False)
        # ndimage.map_coordinates returns a flat array,
        # it needs to be reshaped to the original shape
        V.shape = output_shape
        return V



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
In this example, we create a regression model for an event-related design in
which the response to an event at time T[i] is modeled as depending on the
amount of time since the last stimulus T[i-1]
"""

import numpy as np
import sympy

from nipy.algorithms.statistics.api import Formula, make_recarray
from nipy.modalities.fmri import utils, hrf

# Inter-stimulus intervals (time between events)
dt = np.random.uniform(low=0, high=2.5, size=(50,))
# Onset times from the ISIs
t = np.cumsum(dt)

# We're going to model the amplitudes ('a') by dt (the time between events)
a = sympy.Symbol('a')
linear = utils.define('linear', utils.events(t, dt, f=hrf.glover))
quadratic = utils.define('quad', utils.events(t, dt, f=hrf.glover, g=a**2))
cubic = utils.define('cubic', utils.events(t, dt, f=hrf.glover, g=a**3))

f1 = Formula([linear, quadratic, cubic])

# Evaluate this time-based formula at specific times to make the design matrix
tval = make_recarray(np.linspace(0,100, 1001), 't')
X1 = f1.design(tval, return_float=True)

# Now we make a model where the relationship of time between events and signal
# is an exponential with a time constant tau
l = sympy.Symbol('l')
exponential = utils.events(t, dt, f=hrf.glover, g=sympy.exp(-l*a))
f3 = Formula([exponential])

# Make a design matrix by passing in time and required parameters
params = make_recarray([(4.5, 3.5)], ('l', '_b0'))
X3 = f3.design(tval, params, return_float=True)

# the columns or d/d_b0 and d/dl
tt = tval.view(np.float)
v1 = np.sum([hrf.glovert(tt - s)*np.exp(-4.5*a) for s,a  in zip(t, dt)], 0)
v2 = np.sum([-3.5*a*hrf.glovert(tt - s)*np.exp(-4.5*a) for s,a  in zip(t, dt)], 0)

V = np.array([v1,v2]).T
W = V - np.dot(X3, np.dot(np.linalg.pinv(X3), V))
np.testing.assert_almost_equal((W**2).sum() / (V**2).sum(), 0)



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function # Python 2/3 compatibility
__doc__ = """
This script generates a noisy multi-subject activation image dataset
and applies the Bayesian structural analysis on it

Requires matplotlib

Author : Bertrand Thirion, 2009-2011
"""
print(__doc__)

import numpy as np
import scipy.stats as st

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise RuntimeError("This script needs the matplotlib library")

import nipy.labs.utils.simul_multisubject_fmri_dataset as simul
import nipy.labs.spatial_models.bayesian_structural_analysis as bsa
from nipy.labs.spatial_models.discrete_domain import domain_from_binary_array


def make_bsa_2d(betas, theta=3., dmax=5., ths=0, thq=0.5, smin=0,
                method='simple', verbose=0):
    """
    Function for performing bayesian structural analysis
    on a set of images.

    Parameters
    ----------
    betas, array of shape (nsubj, dimx, dimy) the data used
           Note that it is assumed to be a t- or z-variate
    theta=3., float,
              first level threshold of betas
    dmax=5., float, expected between subject variability
    ths=0, float,
           null hypothesis for the prevalence statistic
    thq=0.5, float,
             p-value of the null rejection
    smin=0, int,
            threshold on the nu_mber of contiguous voxels
            to make regions meaningful structures
    method= 'simple', string,
            estimation method used ; to be chosen among
            'simple', 'quick', 'loo', 'ipmi'
    verbose=0, verbosity mode

    Returns
    -------
    AF the landmark_regions instance describing the result
    BF: list of hroi instances describing the individual data
    """
    ref_dim = np.shape(betas[0])
    nsubj = betas.shape[0]
    xyz = np.array(np.where(betas[:1])).T.astype(np.int)

    # Get  coordinates in mm
    xyz = xyz[:, 1:] # switch to dimension 2
    coord = xyz.astype(np.float)

    # get the functional information
    lbeta = np.array([np.ravel(betas[k]) for k in range(nsubj)]).T

    lmax = 0
    bdensity = 1
    dom = domain_from_binary_array(np.ones(ref_dim))

    if method == 'simple':
        group_map, AF, BF, likelihood = \
                   bsa.compute_BSA_simple(dom, lbeta, dmax, thq, smin, ths,
                                          theta)
    if method == 'quick':
        likelihood = np.zeros(ref_dim)
        group_map, AF, BF, coclustering = \
                   bsa.compute_BSA_quick(dom, lbeta, dmax, thq, smin, ths,
                                         theta)
    if method == 'loo':
        mll, ll0 = bsa.compute_BSA_loo(dom, lbeta, dmax, thq, smin, ths,
                                          theta, bdensity)
        return mll, ll0

    if method not in['loo', 'simple', 'quick']:
        raise ValueError('method is not correctly defined')

    if verbose == 0:
        return AF, BF

    if AF != None:
        lmax = AF.k + 2
        AF.show()

    group_map.shape = ref_dim
    plt.figure(figsize=(8, 3))
    ax = plt.subplot(1, 3, 1)
    plt.imshow(group_map, interpolation='nearest', vmin=-1, vmax=lmax)
    plt.title('Blob separation map', fontsize=10)
    plt.axis('off')
    plt.colorbar(shrink=.8)

    if AF != None:
        group_map = AF.map_label(coord, 0.95, dmax)
        group_map.shape = ref_dim

    plt.subplot(1, 3, 2)
    plt.imshow(group_map, interpolation='nearest', vmin=-1, vmax=lmax)
    plt.title('group-level position 95% \n confidence regions', fontsize=10)
    plt.axis('off')
    plt.colorbar(shrink=.8)

    plt.subplot(1, 3, 3)
    likelihood.shape = ref_dim
    plt.imshow(likelihood, interpolation='nearest')
    plt.title('Spatial density under h1', fontsize=10)
    plt.axis('off')
    plt.colorbar(shrink=.8)


    fig_output = plt.figure(figsize=(8, 3.5))
    fig_output.text(.5, .9, "Individual landmark regions", ha="center")
    for s in range(nsubj):
        ax = plt.subplot(nsubj / 5, 5, s + 1)
        #ax.set_position([.02, .02, .96, .96])
        lw = - np.ones(ref_dim)
        if BF[s] is not None:
            nls = BF[s].get_roi_feature('label')
            nls[nls == - 1] = np.size(AF) + 2
            for k in range(BF[s].k):
                np.ravel(lw)[BF[s].label == k] = nls[k]

        plt.imshow(lw, interpolation='nearest', vmin=-1, vmax=lmax)
        plt.axis('off')

    fig_input = plt.figure(figsize=(8, 3.5))
    fig_input.text(.5,.9, "Input activation maps", ha='center')
    for s in range(nsubj):
        plt.subplot(nsubj / 5, 5, s + 1)
        plt.imshow(betas[s], interpolation='nearest', vmin=betas.min(),
                  vmax=betas.max())
        plt.axis('off')
    return AF, BF


###############################################################################
# Main script
###############################################################################

# generate the data
n_subj = 10
shape = (60, 60)
pos = np.array([[12, 14],
                [20, 20],
                [30, 20]])
ampli = np.array([5, 7, 6])
sjitter = 1.0
betas = simul.surrogate_2d_dataset(n_subj=n_subj, shape=shape, pos=pos,
                                   ampli=ampli, width=5.0)

# set various parameters
theta = float(st.t.isf(0.01, 100))
dmax = 4. / 1.5
ths = n_subj / 4
thq = 0.9
verbose = 1
smin = 5
method = 'simple' # 'quick' #  'loo' #

# run the algo
AF, BF = make_bsa_2d(betas, theta, dmax, ths, thq, smin, method,
                     verbose=verbose)
plt.show()



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import unittest

import numpy as np

from .. import permutation_test as pt
from nipy.algorithms.graph import wgraph_from_3d_grid

nperms = 2
ndraws = 10

def make_data(n=10,mask_shape=(10,10,10),axis=0):
    mask = np.zeros(mask_shape,int)
    XYZ = np.array(np.where(mask==0))
    p = XYZ.shape[1]
    data = np.random.randn(n,p)
    vardata = np.random.randn(n,p)**2
    if axis==1:
        data = data.T
        vardata = vardata.T
    return data, vardata, XYZ

class test_permutation_test(unittest.TestCase):

    def test_onesample(self):
        data, vardata, XYZ = make_data()
        # rfx calibration
        P = pt.permutation_test_onesample(data, XYZ, ndraws=ndraws)
        c = [(P.random_Tvalues[P.ndraws*(0.95)],None), (
                P.random_Tvalues[P.ndraws*(0.5)], 18.)]
        r = np.ones(data.shape[1],int)
        r[data.shape[1]/2:] *= 10
        #p_values, cluster_results, region_results = P.calibrate(nperms=100, clusters=c, regions=[r])
        # mfx calibration
        P = pt.permutation_test_onesample(
            data, XYZ, vardata=vardata, stat_id="student_mfx", ndraws=ndraws)
        p_values, cluster_results, region_results = P.calibrate(
            nperms=nperms, clusters=c, regions=[r])

    def test_onesample_graph(self):
        data, vardata, XYZ = make_data()
        G = wgraph_from_3d_grid(XYZ.T)
        # rfx calibration
        P = pt.permutation_test_onesample_graph(data, G, ndraws=ndraws)
        c = [(P.random_Tvalues[P.ndraws*(0.95)],None)]
        r = np.ones(data.shape[1],int)
        r[data.shape[1]/2:] *= 10
        #p_values, cluster_results, region_results = P.calibrate(nperms=100, clusters=c, regions=[r])
        # mfx calibration
        P = pt.permutation_test_onesample_graph(
            data, G, vardata=vardata, stat_id="student_mfx", ndraws=ndraws)
        p_values, cluster_results, region_results = P.calibrate(
            nperms=nperms, clusters=c, regions=[r])

    def test_twosample(self):
        data, vardata, XYZ = make_data(n=20)
        data1, vardata1, data2, vardata2 = (
            data[:10], vardata[:10], data[10:],vardata[10:])
        # rfx calibration
        P = pt.permutation_test_twosample(data1, data2, XYZ, ndraws=ndraws)
        c = [(P.random_Tvalues[P.ndraws*(0.95)],None),(P.random_Tvalues[P.ndraws*(0.5)],10)]
        r = [np.zeros(data.shape[1])]
        # Assuming our data.shape[1] is divisible by 2
        r[data.shape[1]//2:] *= 10
        #p_values, cluster_results, region_results=P.calibrate(nperms=100, clusters=c, regions=r)
        # mfx calibration
        P = pt.permutation_test_twosample(data1, data2, XYZ, vardata1=vardata1, vardata2=vardata2, stat_id="student_mfx", ndraws=ndraws)
        p_values, cluster_results, region_results = P.calibrate(nperms=nperms, clusters=c, regions=r)


if __name__ == "__main__":
    unittest.main()



"""
Copy out of the top of sympy.core.compatibility as of
b8aa2de87d537eddde044c13f2eaaab99a5dcfe7

This can be deleted when we depend on sympy 0.7.0 or later
"""
"""
Reimplementations of constructs introduced in later versions of Python than we
support.
"""

# These are in here because telling if something is an iterable just by calling
# hasattr(obj, "__iter__") behaves differently in Python 2 and Python 3.  In
# particular, hasattr(str, "__iter__") is False in Python 2 and True in Python 3.
# I think putting them here also makes it easier to use them in the core.

def iterable(i, exclude=(basestring, dict)):
    """
    Return a boolean indicating whether i is an iterable in the sympy sense.

    When sympy is working with iterables, it is almost always assuming
    that the iterable is not a string or a mapping, so those are excluded
    by default. If you want a pure python definition, make exclude=None. To
    exclude multiple items, pass them as a tuple.

    See also: is_sequence

    Examples:

    >>> things = [[1], (1,), set([1]), (j for j in [1, 2]), {1:2}, '1', 1]

    In the test below, Python 2 prints output including e.g ``<type 'list'``
    where Python 3 prints e.g. ``<class 'list'>``.

    >>> for i in things:                    #doctest: +IGNORE_OUTPUT
    ...     print iterable(i), type(i)
    True <type 'list'>
    True <type 'tuple'>
    True <type 'set'>
    True <type 'generator'>
    False <type 'dict'>
    False <type 'str'>
    False <type 'int'>

    >>> iterable({}, exclude=None)
    True
    >>> iterable({}, exclude=str)
    True
    >>> iterable("no", exclude=str)
    False

    """
    try:
        iter(i)
    except TypeError:
        return False
    if exclude:
        return not isinstance(i, exclude)
    return True

def is_sequence(i, include=None):
    """
    Return a boolean indicating whether i is a sequence in the sympy
    sense. If anything that fails the test below should be included as
    being a sequence for your application, set 'include' to that object's
    type; multiple types should be passed as a tuple of types.

    Note: although generators can generate a sequence, they often need special
    handling to make sure their elements are captured before the generator is
    exhausted, so these are not included by default in the definition of a
    sequence.

    See also: iterable

    Examples:

    >>> from nipy.fixes.sympy.utilities.compatibility import is_sequence
    >>> from types import GeneratorType
    >>> is_sequence([])
    True
    >>> is_sequence(set())
    False
    >>> is_sequence('abc')
    False
    >>> is_sequence('abc', include=str)
    True
    >>> generator = (c for c in 'abc')
    >>> is_sequence(generator)
    False
    >>> is_sequence(generator, include=(str, GeneratorType))
    True

    """
    return (hasattr(i, '__getitem__') and
            iterable(i) or
            bool(include) and
            isinstance(i, include))



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
""" Module implements the Forest class

A Forest is a graph with a hierarchical structure.  Each connected component of
a forest is a tree.  The main characteristic is that each node has a single
parent, so that a Forest is fully characterized by a "parent" array, that
defines the unique parent of each node.  The directed relationships are encoded
by the weight sign.

Note that some methods of WeightedGraph class (e.g. dijkstra's algorithm)
require positive weights, so that they cannot work on forests in the current
implementation. Specific methods (e.g. all_sidtance()) have been set instead.

Main author: Bertrand thirion, 2007-2011
"""

import numpy as np

from .graph import WeightedGraph


class Forest(WeightedGraph):
    """ Forest structure, i.e. a set of trees

    The nodes can be segmented into trees.

    Within each tree a node has one parent and children
    that describe the associated hierarchical structure.
    Some of the nodes can be viewed as leaves, other as roots
    The edges within a tree are associated with a weight:

    * +1 from child to parent
    * -1 from parent to child

    Attributes
    ----------
    V : int
        int > 0, the number of vertices
    E : int
        the number of edges
    parents : (self.V,) array
        the parent array
    edges : (self.E, 2) array
        representing pairwise neighbors
    weights : (self.E,) array
        +1/-1 for ascending/descending links
    children: list
        list of arrays that represents the children any node
    """

    def __init__(self, V, parents=None):
        """Constructor

        Parameters
        ----------
        V : int
            the number of edges of the graph
        parents : None or (V,) array
            the parents of zach vertex.  If `parents`==None , the parents are
            set to range(V), i.e. each node is its own parent, and each node is
            a tree
        """
        V = int(V)
        if V < 1:
            raise ValueError('cannot create graphs with no vertex')
        self.V = int(V)

        # define the parents
        if parents == None:
            self.parents = np.arange(self.V).astype(np.int)
        else:
            if np.size(parents) != V:
                raise ValueError('Incorrect size for parents')
            if parents.max() > self.V:
                raise ValueError('Incorrect value for parents')

            self.parents = np.reshape(parents, self.V).astype(np.int)

        self.define_graph_attributes()

        if self.check() == 0:
            raise ValueError('The proposed structure is not a forest')
        self.children = []

    def define_graph_attributes(self):
        """define the edge and weights array
        """
        self.edges = np.array([]).astype(np.int)
        self.weights = np.array([])
        i = np.nonzero(self.parents != np.arange(self.V))[0]
        if np.size(i) > 0:
            E1 = np.hstack((i, self.parents[i]))
            E2 = np.hstack((self.parents[i], i))
            self.edges = (np.vstack((E1, E2))).astype(np.int).T
            self.weights = np.hstack((np.ones(np.size(i)),
                                      - np.ones(np.size(i))))

        self.E = np.size(self.weights)
        self.edges = self.edges

    def compute_children(self):
        """Define the children of each node (stored in self.children)
        """
        self.children = [np.array([]) for v in range(self.V)]
        if self.E > 0:
            K = self.copy()
            K.remove_edges(K.weights < 0)
            self.children = K.to_coo_matrix().tolil().rows.tolist()

    def get_children(self, v=-1):
        """ Get the children of a node/each node

        Parameters
        ----------
        v: int, optional
            a node index

        Returns
        -------
        children: list of int the list of children of node v (if v is provided)
                  a list of lists of int, the children of all nodes otherwise
        """
        v = int(v)
        if v > -1:
            if v > self.V - 1:
                raise ValueError('the given node index is too high')
        if self.children == []:
            self.compute_children()
        if v == -1:
            return self.children
        else:
            return self.children[v]

    def get_descendants(self, v, exclude_self=False):
        """returns the nodes that are children of v as a list

        Parameters
        ----------
        v: int, a node index

        Returns
        -------
        desc: list of int, the list of all descendant of the input node
        """
        v = int(v)
        if v < 0:
            raise ValueError('the given node index is too low')
        if v > self.V - 1:
            raise ValueError('the given node index is too high')
        if self.children == []:
            self.compute_children()
        if len(self.children[v]) == 0:
            return [v]
        else:
            desc = [v]
            for w in self.children[v]:
                temp = self.get_descendants(w)
                for q in temp:
                    desc.append(q)
        desc.sort()
        if exclude_self and v in desc:
            desc = [i for i in desc if i != v]
        return desc

    def check(self):
        """Check that self is indeed a forest, i.e. contains no loop

        Returns
        -------
        a boolean b=0 iff there are loops, 1  otherwise

        Notes
        -----
        Slow implementation, might be rewritten in C or cython
        """
        b = 1
        if self.V == 1:
            return b
        for v in range(self.V):
            w = v
            q = 0
            while(self.parents[w] != w):
                w = self.parents[w]
                if w == v:
                    b = 0
                    break
                q += 1
                if q > self.V:
                    b = 0
                    break
            if b == 0:
                break
        return b

    def isleaf(self):
        """ Identification of the leaves of the forest

        Returns
        -------
        leaves: bool array of shape(self.V), indicator of the forest's leaves
        """
        leaves = np.ones(self.V).astype('bool')
        if self.E > 0:
            leaves[self.edges[self.weights > 0, 1]] = 0
        return leaves

    def isroot(self):
        """ Returns an indicator of nodes being roots

        Returns
        -------
        roots, array of shape(self.V, bool), indicator of the forest's roots
        """
        roots = np.array(self.parents == np.arange(self.V))
        return roots

    def subforest(self, valid):
        """ Creates a subforest with the vertices for which valid > 0

        Parameters
        ----------
        valid: array of shape (self.V): idicator of the selected nodes

        Returns
        -------
        subforest: a new forest instance, with a reduced set of nodes

        Notes
        -----
        The children of deleted vertices become their own parent
        """
        if np.size(valid) != self.V:
            raise ValueError("incompatible size for self anf valid")

        parents = self.parents.copy()
        j = np.nonzero(valid[self.parents] == 0)[0]
        parents[j] = j
        parents = parents[valid.astype(bool)]
        renumb = np.hstack((0, np.cumsum(valid)))
        parents = renumb[parents]

        F = Forest(np.sum(valid), parents)

        return F

    def merge_simple_branches(self):
        """ Return a subforest, where chained branches are collapsed

        Returns
        -------
        sf, Forest instance, same as self, without any chain
        """
        valid = np.ones(self.V).astype('bool')
        children = self.get_children()
        for k in range(self.V):
            if np.size(children[k]) == 1:
                valid[k] = 0
        return self.subforest(valid)

    def all_distances(self, seed=None):
        """returns all the distances of the graph  as a tree

        Parameters
        ----------
        seed=None array of shape(nbseed)  with valuesin [0..self.V-1]
                  set of vertices from which tehe distances are computed

        Returns
        -------
        dg: array of shape(nseed, self.V), the resulting distances

        Notes
        -----
        By convention infinite distances are given the distance np.inf
        """
        if (hasattr(seed, '__iter__') == False) & (seed is not None):
            seed = [seed]

        if self.E > 0:
            w = self.weights.copy()
            self.weights = np.absolute(self.weights)
            dg = self.floyd(seed)
            dg[dg == (np.sum(self.weights) + 1)] = np.inf
            self.weights = w
            return dg
        else:
            return np.inf * np.ones((self.V, self.V))

    def depth_from_leaves(self):
        """compute an index for each node: 0 for the leaves, 1 for
        their parents etc. and maximal for the roots.

        Returns
        -------
        depth: array of shape (self.V): the depth values of the vertices
        """
        depth = self.isleaf().astype(np.int)-1
        for j in range(self.V):
            dc = depth.copy()
            for i in range(self.V):
                if self.parents[i] != i:
                    depth[self.parents[i]] = np.maximum(depth[i] + 1,\
                                           depth[self.parents[i]])
            if dc.max() == depth.max():
                break
        return depth

    def reorder_from_leaves_to_roots(self):
        """reorder the tree so that the leaves come first then their
        parents and so on, and the roots are last.

        Returns
        -------
        order: array of shape(self.V)
               the order of the old vertices in the reordered graph
        """
        depth = self.depth_from_leaves()
        order = np.argsort(depth)
        iorder = np.arange(self.V)
        for i in range(self.V):
            iorder[order[i]] = i
        parents = iorder[self.parents[order]]
        self.parents = parents
        self.define_graph_attributes()
        return order

    def leaves_of_a_subtree(self, ids, custom=False):
        """tests whether the given nodes are the leaves of a certain subtree

        Parameters
        ----------
        ids: array of shape (n) that takes values in [0..self.V-1]
        custom == False, boolean
               if custom==true the behavior of the function is more specific
               - the different connected components are considered
               as being in a same greater tree
               - when a node has more than two subbranches,
               any subset of these children is considered as a subtree
        """
        leaves = self.isleaf().astype('bool')
        for i in ids:
            if leaves[i] == 0:
                raise ValueError("some of the ids are not leaves")

        #1. find the highest node that is a common ancestor to all leaves
        # if there is none, common ancestor is -1
        com_ancestor = ids[0]
        for i in ids:
            ca = i
            dca = self.get_descendants(ca)
            while com_ancestor not in dca:
                ca = self.parents[ca]
                dca = self.get_descendants(ca)
                if (ca == self.parents[ca]) & (com_ancestor not in dca):
                    ca = -1
                    break
            com_ancestor = ca

        #2. check whether all the children of this ancestor are within ids
        if com_ancestor > -1:
            st = self.get_descendants(com_ancestor)
            valid = [i in ids for i in st if leaves[i]]
            bresult = (np.sum(valid) == np.size(valid))
            if custom == False:
                return bresult

            # now, custom =True
            # check that subtrees of ancestor are consistently labelled
            kids = self.get_children(com_ancestor)
            if np.size(kids) > 2:
                bresult = True
                for v in kids:
                    st = np.array(self.get_descendants(v))
                    st = st[leaves[st]]
                    if np.size(st) > 1:
                        valid = [i in ids for i in st]
                        bresult *= ((np.sum(valid) == np.size(valid))
                                    + np.sum(valid == 0))
            return bresult

        # now, common ancestor is -1
        if custom == False:
            st = np.squeeze(np.nonzero(leaves))
            valid = [i in ids for i in st]
            bresult = (np.sum(valid) == np.size(valid))
        else:
            cc = self.cc()
            bresult = True
            for i in ids:
                st = np.squeeze(np.nonzero((cc == cc[i]) * leaves))
                if np.size(st) > 1:
                    valid = [i in ids for i in st]
                    bresult *= (np.sum(valid) == np.size(valid))
                else:
                    bresult *= (st in ids)
        return bresult

    def tree_depth(self):
        """ Returns the number of hierarchical levels in the tree
        """
        depth = self.depth_from_leaves()
        return depth.max() + 1

    def propagate_upward_and(self, prop):
        """propagates from leaves to roots some binary property of the nodes
        so that prop[parents] = logical_and(prop[children])

        Parameters
        ----------
        prop, array of shape(self.V), the input property

        Returns
        -------
        prop, array of shape(self.V), the output property field
        """
        prop = np.asanyarray(prop).copy()
        if np.size(prop) != self.V:
            raise ValueError("incoherent size for prop")

        prop[self.isleaf() == False] = True

        for j in range(self.tree_depth()):
            for i in range(self.V):
                if prop[i] == False:
                    prop[self.parents[i]] = False

        return prop

    def propagate_upward(self, label):
        """ Propagation of a certain labelling from leves to roots
        Assuming that label is a certain positive integer field
        this propagates these labels to the parents whenever
        the children nodes have coherent properties
        otherwise the parent value is unchanged

        Parameters
        ----------
        label: array of shape(self.V)

        Returns
        -------
        label: array of shape(self.V)
        """
        label = np.asanyarray(label).copy()
        if np.size(label) != self.V:
            raise ValueError("incoherent size for label")

        ch = self.get_children()
        depth = self.depth_from_leaves()
        for j in range(1, depth.max() + 1):
            for i in range(self.V):
                if depth[i] == j:
                    if np.size(np.unique(label[ch[i]])) == 1:
                        label[i] = np.unique(label[ch[i]])
        return label



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
''' plot tsdiffana parameters '''

import numpy as np

import nipy
from .timediff import time_slice_diffs


def plot_tsdiffs(results, axes=None):
    ''' Plotting routine for time series difference metrics

    Requires matplotlib
    
    Parameters
    ----------
    results : dict
       Results of format returned from
       :func:`nipy.algorithms.diagnostics.time_slice_diff`
    
    '''
    import matplotlib.pyplot as plt
    T = len(results['volume_means'])
    S = results['slice_mean_diff2'].shape[1]
    mean_means = np.mean(results['volume_means'])
    scaled_slice_diff = results['slice_mean_diff2'] / mean_means

    if axes is None:
        n_plots = 4
        fig = plt.figure()
        fig.set_size_inches([10,10])
        axes = [plt.subplot(n_plots, 1, i+1) for i in range(n_plots)]
        
    def xmax_labels(ax, val, xlabel, ylabel):
        xlims = ax.axis()
        ax.axis((0, val) + xlims[2:])
        ax.set_xlabel(xlabel)
        ax.set_ylabel(ylabel)
        
    # plot of mean volume variance
    ax = axes[0]
    ax.plot(results['volume_mean_diff2'] / mean_means)
    xmax_labels(ax, T-1, 'Difference image number', 'Scaled variance')
    
    # plot of diff by slice
    ax = axes[1]
    #Set up the color map for the different slices:
    X, Y = np.meshgrid(np.arange(scaled_slice_diff.shape[0]),
                       np.arange(scaled_slice_diff.shape[1]))

    # Use HSV in order to code the slices from bottom to top:
    ax.scatter(X.T.ravel(),scaled_slice_diff.ravel(),
               c=Y.T.ravel(),cmap=plt.cm.hsv,
               alpha=0.2)

    xmax_labels(ax, T-1,
                'Difference image number',
                'Slice by slice variance')
    
    # mean intensity
    ax = axes[2]
    ax.plot(results['volume_means'] / mean_means)
    xmax_labels(ax, T,
                'Image number',
                'Scaled mean \n voxel intensity')
    
    # slice plots min max mean
    ax = axes[3]
    ax.hold(True)
    ax.plot(np.mean(scaled_slice_diff, 0), 'k')
    ax.plot(np.min(scaled_slice_diff, 0), 'b')
    ax.plot(np.max(scaled_slice_diff, 0), 'r')
    ax.hold(False)
    xmax_labels(ax, S+1,
                'Slice number',
                'Max/mean/min \n slice variation')
    return axes


def plot_tsdiffs_image(img, axes=None, show=True):
    ''' Plot time series diagnostics for image

    Parameters
    ----------
    img : image-like or filename str
       image on which to do diagnostics
    axes : None or sequence, optional
       Axes on which to plot the diagnostics.  If None, then we create a
       figure and subplots for the plots.  Sequence should have length
       >=4. 
    show : bool, optional
       If True, show the figure after plotting it

    Returns
    -------
    axes : Matplotlib axes
       Axes on which we have done the plots.   Will be same as `axes`
       input if not None
    '''
    if isinstance(img, basestring):
        title = img
    else:
        title = 'Difference plots'
    img = nipy.as_image(img)
    res = time_slice_diffs(img)
    axes = plot_tsdiffs(res, axes)
    axes[0].set_title(title)
    if show:
        # show the plot
        import matplotlib.pyplot as plt
        plt.show()
    return axes



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function # Python 2/3 compatibility
__doc__ = """
Demo ward clustering on a graph: various ways of forming clusters and dendrogram

Requires matplotlib
"""
print(__doc__)

import numpy as np
from numpy.random import randn, rand

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise RuntimeError("This script needs the matplotlib library")

from nipy.algorithms.graph import knn
from nipy.algorithms.clustering.hierarchical_clustering import ward

# n = number of points, k = number of nearest neighbours
n = 100
k = 5

# Set verbose to True to see more printed output
verbose = False

X = randn(n, 2)
X[:np.ceil(n / 3)] += 3
G = knn(X, 5)
tree = ward(G, X, verbose)

threshold = .5 * n
u = tree.partition(threshold)

plt.figure(figsize=(12, 6))
plt.subplot(1, 3, 1)
for i in range(u.max()+1):
    plt.plot(X[u == i, 0], X[u == i, 1], 'o', color=(rand(), rand(), rand()))

plt.axis('tight')
plt.axis('off')
plt.title('clustering into clusters \n of inertia < %g' % threshold)

u = tree.split(k)
plt.subplot(1, 3, 2)
for e in range(G.E):
    plt.plot([X[G.edges[e, 0], 0], X[G.edges[e, 1], 0]],
            [X[G.edges[e, 0], 1], X[G.edges[e, 1], 1]], 'k')
for i in range(u.max() + 1):
    plt.plot(X[u == i, 0], X[u == i, 1], 'o', color=(rand(), rand(), rand()))
plt.axis('tight')
plt.axis('off')
plt.title('clustering into 5 clusters')

nl = np.sum(tree.isleaf())
validleaves = np.zeros(n)
validleaves[:np.ceil(n / 4)] = 1
valid = np.zeros(tree.V, 'bool')
valid[tree.isleaf()] = validleaves.astype('bool')
nv = np.sum(validleaves)
nv0 = 0
while nv > nv0:
    nv0 = nv
    for v in range(tree.V):
        if valid[v]:
            valid[tree.parents[v]]=1
        nv = np.sum(valid)

ax = plt.subplot(1, 3, 3)
ax = tree.plot(ax)
ax.set_title('Dendrogram')
ax.set_visible(True)
plt.show()

if verbose:
    print('List of sub trees')
    print(tree.list_of_subtrees())



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function # Python 2/3 compatibility

import sys

USAGE = """
usage : python %s [1x4-contrast]
where [1x4-contrast] is optional and is something like 1,0,0,0

If you don't enter a contrast, 1,0,0,0 is the default.

An activation image is displayed.

This script requires the nipy-data package to run. It is an example of using a
general linear model in single-subject fMRI data analysis context. Two sessions
of the same subject are taken from the FIAC'05 dataset.

The script also needs matplotlib installed.

Author: Alexis Roche, Bertrand Thirion, 2009--2012.
""" % sys.argv[0]

__doc__ = USAGE

import numpy as np

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise RuntimeError("This script needs the matplotlib library")

from nipy.labs.viz import plot_map, cm
from nipy.modalities.fmri.glm import FMRILinearModel
from nipy.utils import example_data


# Optional argument - default value 1, 0, 0, 0
nargs = len(sys.argv)
if nargs not in (1, 2, 5):
    print(USAGE)
    exit(1)
if nargs == 1:  # default no-argument case
    cvect = [1, 0, 0, 0]
else:
    if nargs == 2:  # contrast as one string
        args = sys.argv[1].split(',')
    elif nargs == 5:  # contrast as sequence of strings
        args = [arg.replace(',', '') for arg in sys.argv[1:]]
    if len(args) != 4:
        print(USAGE)
        exit(1)
    try:
        cvect = [float(arg) for arg in args]
    except ValueError:
        print(USAGE)
        exit(1)

# Input files
fmri_files = [example_data.get_filename('fiac', 'fiac0', run)
              for run in ['run1.nii.gz', 'run2.nii.gz']]
design_files = [example_data.get_filename('fiac', 'fiac0', run)
                for run in ['run1_design.npz', 'run2_design.npz']]
mask_file = example_data.get_filename('fiac', 'fiac0', 'mask.nii.gz')

# Load all the data
multi_session_model = FMRILinearModel(fmri_files, design_files, mask_file)

# GLM fitting
multi_session_model.fit(do_scaling=True, model='ar1')

# Compute the required contrast
print('Computing test contrast image...')
n_regressors = [np.load(f)['X'].shape[1] for f in design_files]
con = [np.hstack((cvect, np.zeros(nr - len(cvect)))) for nr in n_regressors]
z_map, = multi_session_model.contrast(con)

# Show Z-map image
mean_map = multi_session_model.means[0]
plot_map(z_map.get_data(),
         z_map.get_affine(),
         anat=mean_map.get_data(),
         anat_affine=mean_map.get_affine(),
         cmap=cm.cold_hot,
         threshold=2.5,
         black_bg=True)
plt.show()



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
""" Smoke testing the cm module
"""
from nose import SkipTest

try:
    import matplotlib as mp
    # Make really sure that we don't try to open an Xserver connection.
    mp.use('svg', warn=False)
    import pylab as pl
    pl.switch_backend('svg')
except ImportError:
    raise SkipTest('Could not import matplotlib')

from ..cm import dim_cmap, replace_inside


def test_dim_cmap():
    # This is only a smoke test
    mp.use('svg', warn=False)
    import pylab as pl
    dim_cmap(pl.cm.jet)


def test_replace_inside():
    # This is only a smoke test
    mp.use('svg', warn=False)
    import pylab as pl
    pl.switch_backend('svg')
    replace_inside(pl.cm.jet, pl.cm.hsv, .2, .8)
    # We also test with gnuplot, which is defined using function
    if hasattr(pl.cm, 'gnuplot'):
        # gnuplot is only in recent version of MPL
        replace_inside(pl.cm.gnuplot, pl.cm.gnuplot2, .2, .8)


